import sys
import os
import platform
import itertools
import shutil #for copy
import re #for string replacements
import string
import tempfile
import zipfile
import getpass
import time
import multiprocessing
import urlparse
import urllib2
import contextlib

from _python_compat import *
import WinCCOA

try:
    import pysvn
    useSvn = True
except:
    useSvn = False

###### Meta configuration
allowConfigFile=False
allowDbAgent=False
allowPing=True

class PVSSBootstrapperException(Exception):
    """class PVSSBootstrapperException(Exception) ->
    Custom exception.
    Can be used as a normal exception.
    Or can be constructed with a retInfo (information returned by the PVSS script
    and read from the config file).
    """
    @staticmethod
    def withRetInfo(retInfo):
        inst = PVSSBootstrapperException(retInfo['errMsg'])
        inst.retInfo = retInfo
        return inst

    @staticmethod
    def withRetInfoWarning(retInfo):
        inst = PVSSBootstrapperException(retInfo['warningMsg'])
        inst.retInfo = retInfo
        return inst

def _zipExtractAllSubPath(zipFile, subPath, destPath):
    """_zipExtractAllSubPath(zipFile, subPath, destPath) ->
    Given a ZIP file the function will extract all files of the archive under the subPath.
    """
    for member in filter(lambda m: m.startswith(subPath), zipFile.namelist()):
        if os.path.basename(member) == '': # Folder
            dirname = os.path.dirname(member)
            if dirname == subPath:
                continue
            else:
                os.mkdir(os.path.join(destPath, dirname[len(subPath)+1:len(dirname)]))
        else: # File
            targetPath = os.path.join(destPath, member[len(subPath)+1:len(member)])
            target = file(targetPath, "wb")
            try:
                if 'open' in dir(zipFile): # Since Python 2.6
                    try:
                        source = zipFile.open(member)
                        shutil.copyfileobj(source, target)
                    finally:
                        try:
                            source.close()
                        except UnboundLocalError:
                            pass
                else: # Before Python 2.6, risky (memory limit) with big files in the Zip
                    target.write(zipFile.read(member))
            finally:
                try:
                    target.close()
                except UnboundLocalError:
                    pass

def _cleanNoneParameters(args):
    """_cleanNoneParameters(args) ->
    Used to remove all dictionnary parameters given to an API function that could have the None value.
    """
    return dict(itertools.ifilter(lambda kv: kv[1] != None, args.iteritems()))

#
# Python API for the bootstrapper
#

def _genericOptions(**kwargs):
    """_genericOptions(**kwargs)
    Handle options common to all functions (pvss, debug and warningLevel).
    The dictionnary returned will be used as an argument for the _PVSSBootstrapper class constructor.
    """
    replacement_dict = {}

    pvssVersion = None
    if 'pvss' in kwargs:
        pvssVersion = str(kwargs['pvss'])

    debug = False
    if ('debug' in kwargs) and kwargs['debug']:
        replacement_dict['debug'] = str(kwargs['debug']).lower()
        debug = True

    cleanup = True
    if ('noCleanup' in kwargs) and kwargs['noCleanup']:
        cleanup = False

    warningLevel = 1 # 0=silent, 1=printed, 2=exception raised
    if 'warningLevel' in kwargs:
        warningLevel = int(kwargs['warningLevel'])
    if warningLevel < 0:
        warningLevel = 0
    if warningLevel > 2:
        warningLevel = 2

    genOpts = {
        'replacement_dict' : replacement_dict,
        'debug' : debug,
        'cleanup': cleanup,
        'pvssVersion' : pvssVersion,
        'warningLevel' : warningLevel,
        'tempDirs' : []
    }

    return genOpts

# API: Entry points

def startProject(project, **kwargs):
    """startProject(project, **kwargs) ->
    Start a PVSS project.

    If a path to the project is given instead of just a name and if this project
    is not registered, it will be automatically registered unless you use the 'dontRegister' option.

    If the project is already running it will not restart it unless you use the 'restart' option.

    The Bootstrapper will wait until all the managers in always mode are running.

    This function opens automatically the console and the log viewer unless you
    use the 'noConsole' and 'noLogViewer' options.

    If onlyPmon is true, it will just spawn a pmon for this project but not start it.

    PVSSBootstrapper.startProject(PROJECT,
                                  restart=True,
                                  user=USER, password=PASSWORD,
                                  noConsole=True, noLogViewer=True,
                                  dontRegister=True, dontWait=True
                                  onlyPmon=False)
    """
    kwargs = _cleanNoneParameters(kwargs)
    genOpts = _genericOptions(**kwargs)

    genOpts['replacement_dict']['configSource'] = 'ConfigFile'
    genOpts['replacement_dict']['function'] = 'StartProject'

    absProject = os.path.abspath(project)
    if os.path.exists(os.path.join(absProject, 'config', 'config')):
        genOpts['replacement_dict']['projectPath'] = absProject
    else:
        genOpts['replacement_dict']['projectName'] = project

    if ('restart' in kwargs) and kwargs['restart']:
        genOpts['replacement_dict']['restart'] = str(kwargs['restart']).lower()
    if 'user' in kwargs:
        genOpts['replacement_dict']['user'] = str(kwargs['user'])
    if 'password' in kwargs:
        genOpts['replacement_dict']['password'] = str(kwargs['password'])
    if ('noConsole' in kwargs) and kwargs['noConsole']:
        genOpts['replacement_dict']['openConsole'] = str(not kwargs['noConsole']).lower()
    if ('noLogViewer' in kwargs) and kwargs['noLogViewer']:
        genOpts['replacement_dict']['openLogViewer'] = str(not kwargs['noLogViewer']).lower()
    if ('dontRegister' in kwargs) and kwargs['dontRegister']:
        genOpts['replacement_dict']['dontRegister'] = str(kwargs['dontRegister']).lower()
    if ('dontWait' in kwargs) and kwargs['dontWait']:
        genOpts['replacement_dict']['dontWait'] = str(kwargs['dontWait']).lower()
    if ('onlyPmon' in kwargs) and kwargs['onlyPmon']:
        genOpts['replacement_dict']['onlyPmon'] = str(kwargs['onlyPmon']).lower()

    _execBootstrapper(**genOpts)

def stopProject(projectName, **kwargs):
    kwargs = _cleanNoneParameters(kwargs)
    genOpts = _genericOptions(**kwargs)

    genOpts['replacement_dict']['configSource'] = 'ConfigFile'
    genOpts['replacement_dict']['function'] = 'StopProject'

    genOpts['replacement_dict']['projectName'] = projectName

    _execBootstrapper(**genOpts)

def stopAllProjects(**kwargs):
    kwargs = _cleanNoneParameters(kwargs)
    genOpts = _genericOptions(**kwargs)

    genOpts['replacement_dict']['configSource'] = 'ConfigFile'
    genOpts['replacement_dict']['function'] = 'StopAllProjects'

    _execBootstrapper(**genOpts)

def isProjectRunning(projectName, **kwargs):
    kwargs = _cleanNoneParameters(kwargs)
    genOpts = _genericOptions(**kwargs)

    genOpts['replacement_dict']['configSource'] = 'ConfigFile'
    genOpts['replacement_dict']['function'] = 'IsProjectRunning'

    genOpts['replacement_dict']['projectName'] = projectName

    retInfo = _execBootstrapper(**genOpts)

    isRunning = retInfo['returnData']
    if isRunning == 'True':
        return True
    elif isRunning == 'False':
        return False
    else:
        raise PVSSBootstrapperException('Returned data is invalid (not \'True\' or \'False\'): ' + isRunning)

def registerProject(projectPath, **kwargs):
    kwargs = _cleanNoneParameters(kwargs)
    genOpts = _genericOptions(**kwargs)

    genOpts['replacement_dict']['configSource'] = 'ConfigFile'
    genOpts['replacement_dict']['function'] = 'RegisterProject'

    genOpts['replacement_dict']['projectPath'] = os.path.abspath(projectPath)

    if 'pmonPort' in kwargs:
        genOpts['replacement_dict']['pmonPort'] = int(kwargs['pmonPort'])

    _execBootstrapper(**genOpts)

def unregisterProject(projectName, **kwargs):
    kwargs = _cleanNoneParameters(kwargs)
    genOpts = _genericOptions(**kwargs)

    genOpts['replacement_dict']['configSource'] = 'ConfigFile'
    genOpts['replacement_dict']['function'] = 'UnregisterProject'

    genOpts['replacement_dict']['projectName'] = projectName

    if ('dontStop' in kwargs) and kwargs['dontStop']:
        genOpts['replacement_dict']['dontStop'] = str(kwargs['dontStop']).lower()

    _execBootstrapper(**genOpts)

def deleteProject(projectName, **kwargs):
    kwargs = _cleanNoneParameters(kwargs)
    genOpts = _genericOptions(**kwargs)

    genOpts['replacement_dict']['configSource'] = 'ConfigFile'
    genOpts['replacement_dict']['function'] = 'DeleteProject'

    genOpts['replacement_dict']['projectName'] = projectName

    if ('dontStop' in kwargs) and kwargs['dontStop']:
        genOpts['replacement_dict']['dontStop'] = str(kwargs['dontStop']).lower()

    _execBootstrapper(**genOpts)

def isProjectRegistered(projectName, **kwargs):
    kwargs = _cleanNoneParameters(kwargs)
    genOpts = _genericOptions(**kwargs)

    genOpts['replacement_dict']['configSource'] = 'ConfigFile'
    genOpts['replacement_dict']['function'] = 'IsProjectRegistered'

    genOpts['replacement_dict']['projectName'] = projectName

    retInfo = _execBootstrapper(**genOpts)

    isRegistered = retInfo['returnData']
    if isRegistered == 'True':
        return True
    elif isRegistered == 'False':
        return False
    else:
        raise PVSSBootstrapperException('Returned data is invalid (not \'True\' or \'False\'): ' + isRegistered)

def createProject(installPath, projectName, **kwargs):
    kwargs = _cleanNoneParameters(kwargs)
    genOpts = _genericOptions(**kwargs)

    genOpts['replacement_dict']['configSource'] = 'ConfigFile'
    genOpts['replacement_dict']['function'] = 'CreateProject'

    genOpts['replacement_dict']['installPath'] = os.path.abspath(installPath)
    genOpts['replacement_dict']['projectName'] = projectName

    if 'systemNumber' in kwargs:
        genOpts['replacement_dict']['sysNum'] = int(kwargs['systemNumber'])
    if 'systemName' in kwargs:
        genOpts['replacement_dict']['sysName'] = str(kwargs['systemName'])
    if ('overwrite' in kwargs) and kwargs['overwrite']:
        genOpts['replacement_dict']['overwrite'] = str(kwargs['overwrite']).lower()
    if 'pmonPort' in kwargs:
        genOpts['replacement_dict']['pmonPort'] = int(kwargs['pmonPort'])
    if 'dataPort' in kwargs:
        genOpts['replacement_dict']['dataPort'] = int(kwargs['dataPort'])
    if 'eventPort' in kwargs:
        genOpts['replacement_dict']['eventPort'] = int(kwargs['eventPort'])
    if 'distPort' in kwargs:
        genOpts['replacement_dict']['distPort'] = int(kwargs['distPort'])

    _execBootstrapper(**genOpts)

def getProjectPath(projectName, **kwargs):
    kwargs = _cleanNoneParameters(kwargs)
    genOpts = _genericOptions(**kwargs)

    genOpts['replacement_dict']['configSource'] = 'ConfigFile'
    genOpts['replacement_dict']['function'] = 'GetProjectPath'

    genOpts['replacement_dict']['projectName'] = projectName

    retInfo = _execBootstrapper(**genOpts)

    projPath = retInfo['returnData']
    if (projPath != None) and (projPath != ''):
        return projPath
    else:
        raise PVSSBootstrapperException('Returned data is invalid: no data')

def listProjects(**kwargs):
    kwargs = _cleanNoneParameters(kwargs)
    genOpts = _genericOptions(**kwargs)

    genOpts['replacement_dict']['configSource'] = 'ConfigFile'
    genOpts['replacement_dict']['function'] = 'ListProjects'

    retInfo = _execBootstrapper(**genOpts)

    projsList = retInfo['returnData']
    if projsList != None:
        return projsList.split('/')
    else:
        return []

def deployZip(projectName, zipFile, **kwargs):
    kwargs = _cleanNoneParameters(kwargs)
    genOpts = _genericOptions(**kwargs)

    genOpts['replacement_dict']['configSource'] = 'ConfigFile'
    genOpts['replacement_dict']['function'] = 'DeployZip'

    genOpts['replacement_dict']['projectName'] = projectName

    zipFilePath = os.path.abspath(zipFile)
    if(not os.path.exists(zipFilePath)):
        raise PVSSBootstrapperException('Cannot find absolute path of ' + zipFile + '.')
    genOpts['replacement_dict']['zipFile'] = zipFilePath

    _execBootstrapper(**genOpts)

def setComponentInstallationFolder(projectName, installDir, **kwargs):
    kwargs = _cleanNoneParameters(kwargs)
    genOpts = _genericOptions(**kwargs)

    genOpts['replacement_dict']['configSource'] = 'ConfigFile'
    genOpts['replacement_dict']['function'] = 'SetCompInstDir'

    genOpts['replacement_dict']['projectName'] = projectName
    genOpts['replacement_dict']['componentInstallDir'] = installDir

    _execBootstrapper(**genOpts)

def installComponent(projectName, sources, components, **kwargs):
    kwargs = _cleanNoneParameters(kwargs)
    genOpts = _genericOptions(**kwargs)

    genOpts['replacement_dict']['configSource'] = 'ConfigFile'
    genOpts['replacement_dict']['function'] = 'InstallComponent'

    genOpts['replacement_dict']['projectName'] = projectName

    def _make_config_line(key, value):
        return '%s = "%s"' % (key, value)

    if not hasattr(components, '__iter__'):
        components = [components]
    if not all(map(lambda c: '.xml' in c, components)):
        raise PVSSBootstrapperException('Component names should be in the form \'component.xml\'.')

    genOpts['replacement_dict']['installComponents'] = '\n'.join(map(lambda comp: _make_config_line('installComponents', comp), components))

    if not hasattr(sources, '__iter__'):
        sources = [sources]

    def getLogin(realm, username, may_save):
        print 'Authentication realm: ' + realm

        newUsername = raw_input('Username [' + username + ']: ')
        if newUsername != '':
            username = newUsername

        password = getpass.getpass('Password: ')
        return True, username, password, may_save

    if useSvn:
        svnClient = pysvn.Client()
        svnClient.callback_get_login = getLogin

    sourcesLocal = []
    for source in sources:
        # Zip file
        if zipfile.is_zipfile(source):
            tempDir = tempfile.mkdtemp(prefix='PVSSBootstrapperZIP-')
            genOpts['tempDirs'] += [tempDir]

            zipFile = zipfile.ZipFile(source, 'r')
            if any(map(lambda m: m.startswith('/') or m.startswith('../'), zipFile.namelist())):
                raise PVSSBootstrapperException('Zip file ' + source + ' contains files starting with \'/\' or \'../\'.')

            # Detect where the first XML files are in the tree
            xmlBase = []
            for member in zipFile.namelist():
                if member.endswith('.xml'):
                    if os.path.dirname(member) not in xmlBase:
                        xmlBase.append(os.path.dirname(member))

            _zipExtractAllSubPath(zipFile, min(xmlBase), tempDir)

            sourcesLocal += [tempDir]
            zipFile.close()

        # SVN repository
        elif ('http://' in source) or ('https://' in source) or ('svn+ssh://' in source):
            if not useSvn:
                raise PVSSBootstrapperException('You need to install python-svn (or pysvn) to use SVN sources.')

            tempDir = tempfile.mkdtemp(prefix='PVSSBootstrapperSVN-')
            genOpts['tempDirs'] += [tempDir]

            shutil.rmtree(tempDir)
            rev = svnClient.export(source, tempDir)
            if genOpts['debug']:
                print 'Exported ' + source + ' at revision ' + str(rev.number)

            sourcesLocal += [tempDir]

        # Local directory
        elif os.path.exists(source) and os.path.isdir(source):
            #foundDirs = []
            #for root, dirs, files in os.walk(source):
                #xmlRegex = re.compile("\.xml$", re.IGNORECASE)
                #files = filter(xmlRegex.search, files)
                #if files:
                    #print os.path.abspath(root)
                    #sourcesLocal += [os.path.abspath(root)]#
                    #foundDirs.append(os.path.abspath(root))
            #sourcesLocal += [foundDirs]

            sourcesLocal += [os.path.abspath(source)]

        else:
            raise PVSSBootstrapperException('"' + source + '" is not a valid component source. Only existing local directory, zip file or SVN repository are accepted.')

    genOpts['replacement_dict']['componentSources'] = '\n'.join(map(lambda source: _make_config_line('componentSources', source), sourcesLocal))

    _execBootstrapper(**genOpts)

# Refer to allowDbAgent meta configuration variable
def dbAgent(db, user, encPassword, **kwargs):
    kwargs = _cleanNoneParameters(kwargs)
    genOpts = _genericOptions(**kwargs)

    genOpts['replacement_dict']['configSource'] = 'ConfDBSysInfo'
    genOpts['replacement_dict']['execMode'] = 'Agent'
    genOpts['replacement_dict']['configDb'] = db
    genOpts['replacement_dict']['configDbUser'] = db_user
    genOpts['replacement_dict']['configDbEncPwd'] = encPassword

    if 'refreshPeriod' in kwargs:
        genOpts['replacement_dict']['agentSleep'] = str(kwargs['refreshPeriod'])

    _execBootstrapper(**genOpts)

# Refer to allowConfigFile meta configuration variable
def configFile(configFile, **kwargs):
    kwargs = _cleanNoneParameters(kwargs)
    genOpts = _genericOptions(**kwargs)

    genOpts['replacement_dict']['configFile'] = os.path.abspath(config_file)

    _execBootstrapper(**genOpts)

# Refer to allowPing meta configuration variable.
# Ping function is for test purpose.
def ping(**kwargs):
    kwargs = _cleanNoneParameters(kwargs)
    genOpts = _genericOptions(**kwargs)

    genOpts['replacement_dict']['configSource'] = 'ConfigFile'
    genOpts['replacement_dict']['function'] = 'Ping'

    retInfo = _execBootstrapper(**genOpts)

    return retInfo['returnData']

def _execBootstrapper(**kwargs):
    bootstrapperInstance = _PVSSBootstrapper(
        replacement_dict=kwargs['replacement_dict'],
        debug=kwargs['debug'],
        cleanup=kwargs['cleanup'],
        wccoaVersion=kwargs['pvssVersion'],
        warningLevel=kwargs['warningLevel'],
        tempDirs=kwargs['tempDirs']
    )

    return bootstrapperInstance.run()

class _PVSSBootstrapper(object):
    def __init__(self, replacement_dict, debug=False, cleanup=True, wccoaVersion=None, warningLevel=1, tempDirs=[]):
        self.replacement_dict = replacement_dict
        self.debug = debug
        self.cleanup = cleanup
        self.tempDirs = tempDirs
        self.warningLevel = warningLevel

        wccoaInstancesDict = dict([(wccoa.version, wccoa) for wccoa in WinCCOA.getInstances()])
        if wccoaVersion:
            if wccoaVersion in wccoaInstancesDict:
                self.wccoa = wccoaInstancesDict[wccoaVersion]
            else:
                raise PVSSBootstrapperException.withRetInfo({
                    'errCode': -1,
                    'errMsg': 'ERROR: WinCC OA version ' + wccoaVersion + ' not available. Only versions available are: ' + str(wccoaInstancesDict.keys())
                })
        else:
            wccoaVersions = sorted(wccoaInstancesDict.keys())
            if wccoaVersions:
                self.wccoa = wccoaInstancesDict[wccoaVersions[-1]] # Get the last one
            else:
                raise PVSSBootstrapperException.withRetInfo({
                    'errCode': -1,
                    'errMsg': 'ERROR: No WinCC OA installation found. Check your registry.'
                })

    def run(self):
        ## TODO:
        ## docstring WCCOA
        try:
            if ('execMode' in self.replacement_dict) and (self.replacement_dict['execMode'] == 'Agent'):
                raise PVSSBootstrapperException.withRetInfo({
                    'errCode': -1,
                    'errMsg': 'ERROR: Agent mode deactivated for the moment to avoid unnecessary dependency on the fwInstallation tool.'
                })

            def preRunHook(tempProjectDir):
                sourceProject = os.path.dirname(__file__)
                self.ctrlConfig = os.path.join(tempProjectDir, 'config', 'config.PVSSBootstrapper')

                shutil.copytree(os.path.join(sourceProject, 'scripts'), os.path.join(tempProjectDir, 'scripts'), ignore=shutil.ignore_patterns('.svn'))
                self.replaceMarkersInConfigFile(os.path.join(sourceProject, 'config', 'config.PVSSBootstrapper.model'), self.ctrlConfig)

            args = []
            if self.debug:
                args.append('log')
                args.append('+stderr')
            args.append('libs/PVSSBootstrapper/PVSSBootstrapper.ctl')

            tempProjectDir = tempfile.mkdtemp(prefix='WCCOABootstrapper-%s-' % self.replacement_dict['function'])
            self.tempDirs += [tempProjectDir]

            tempProjectDir = self.wccoa.runManagerInFakeProject('ctrl', preRunHook, *args, tempProjectDir=tempProjectDir)

            retInfo = self._getReturnInfo()


            if retInfo['errCode'] < 0:
                raise PVSSBootstrapperException.withRetInfo(retInfo)

            if retInfo['warningMsg']:
                if self.warningLevel == 0:
                    pass
                elif self.warningLevel == 1:
                    print 'WARNING: ' + retInfo['warningMsg']
                elif self.warningLevel == 2:
                    retInfo['errCode'] = -1
                    raise PVSSBootstrapperException.withRetInfoWarning(retInfo)

            return retInfo

        finally:
            # Clean-up temp directories
            if self.cleanup:
                for d in self.tempDirs:
                    if os.path.exists(d):
                        self._keepTryingRmTree(d)
            elif self.debug:
                print 'Temporary folders NOT cleaned up:'
                for d in self.tempDirs:
                    if os.path.exists(d):
                        print d

    def _keepTryingRmTree(self, path):
        try:
            shutil.rmtree(path)
        except WindowsError:
            debugFilePath = os.path.join(os.path.dirname(path), os.path.basename(path) + '-debug.txt')
            with open(debugFilePath, 'w') as debugFile:
                for k, v in self.replacement_dict.items():
                    debugFile.write(k + ': ' + v + '\n')
                debugFile.write('\n\n\n')
                import traceback
                debugFile.writelines(traceback.format_stack())

            p = multiprocessing.Process(target=self._winFork, args=(path, debugFilePath))
            p.daemon=True # To allow winfork to get killed when we, the parent process, complete. 150209 PCB
            p.start()

    def _winFork(self, path, debugFilePath):
        while(os.path.exists(path)):
            try:
                shutil.rmtree(path)
            except WindowsError:
                time.sleep(1)

        os.remove(debugFilePath)

    # Replace all replacement-markers
    def replaceMarkersInConfigFile(self, source, dest):
        # Custom dictionnary class returning "%(marker)s" when an element is not defined in the dict
        # It allows afterwards to remove these lines with regexp.
        class DefaultDict(dict):
            def __missing__(self, key):
                return '%(' + str(key) + ')s'

        # Do the processing
        with open(source, 'r') as fileIn:
            with open(dest, 'w') as fileOut:
                configFileContent = fileIn.read()

                configFileContent %= DefaultDict(self.replacement_dict) # Do the python string formatting

                configFileContent = re.sub(".*%(.*).*", u'', configFileContent) # Remove lines where there is still replacement markers in it
                configFileContent = re.sub("#.*", u'', configFileContent) # Remove comments
                configFileContent = re.sub(u'(?imu)^\s*\n', u'', configFileContent) # Remove empty lines

                fileOut.write(configFileContent)


    def _getReturnInfo(self):
        errCode = '-1'
        errMsg = 'No return info found'
        warningMsg = None
        returnData = None

        kVs = _readConfigFile(self.ctrlConfig)
        if 'errCode' in kVs:
            errCode = kVs['errCode']
        if 'errMsg' in kVs:
            errMsg = kVs['errMsg']
        if ('warningMsg' in kVs) and (kVs['warningMsg'] != ''):
            warningMsg = kVs['warningMsg']
        if ('returnData' in kVs) and (kVs['returnData'] != ''):
            returnData = kVs['returnData']

        return {'errCode': int(errCode), 'errMsg': errMsg, 'warningMsg': warningMsg, 'returnData': returnData}

# Out of the class to be usable by test scripts
def _readConfigFile(configFile):
    with open(configFile, "r") as fileOut:
        kVs = re.findall(r'^(.*?) *= *"?(.*?)"?$', fileOut.read(), re.M)

    return dict(kVs)

### Utility functions to grab files over a URL
### Could not use urlgrabber because the current version fails in so many ways and the project seems stalling.

# Custom HTTPS opener for fetching file from EDMS.
# See http://bugs.python.org/issue11220.
# Reason seems to be "banner's oracle 10g server supports SSLv3 only".
# Normal HTTPS opener failing only on some Ubuntu versions though. SLC5 seems ok.
import httplib, ssl, socket
class _HTTPSConnectionV3(httplib.HTTPSConnection):
    def __init__(self, *args, **kwargs):
        httplib.HTTPSConnection.__init__(self, *args, **kwargs)
    def connect(self):
        sock = socket.create_connection((self.host, self.port), self.timeout)
        if self._tunnel_host:
            self.sock = sock
            self._tunnel()
        try:
            self.sock = ssl.wrap_socket(sock, self.key_file, self.cert_file, ssl_version=ssl.PROTOCOL_SSLv3)
        except ssl.SSLError, e:
            print 'Trying SSLv23'
            self.sock = ssl.wrap_socket(sock, self.key_file, self.cert_file, ssl_version=ssl.PROTOCOL_SSLv23)

class _HTTPSHandlerV3(urllib2.HTTPSHandler):
    def https_open(self, req):
        try:
            return self.do_open(httplib.HTTPSConnection, req)
        except urllib2.URLError, e:
            return self.do_open(_HTTPSConnectionV3, req)

urllib2.install_opener(urllib2.build_opener(_HTTPSHandlerV3()))

def grabURLFile(url, destFolder):
    with contextlib.closing(urllib2.urlopen(url)) as urlFile:
        dest = os.path.join(destFolder, os.path.basename(urlparse.urlsplit(urlFile.geturl()).path))
        with open(dest, 'wb') as destFile:
            destFile.write(urlFile.read())

    return dest
