# WinCCOA.py

import os
import re
import string
import platform
import subprocess
import tempfile
import shutil
import sys # For exceptions


# Under Windows, registry entries for WinCC OA are... random.
# So we rather scan the directory structure for things that looks like a WinCC OA isntallation.
PVSS_WIN_PATHS = [
    "C:\\Siemens\\Automation\\WinCC_OA\\",
    "C:\\ETM\\PVSS2\\",
]

class WinCCOA(object):
    def __init__(self, version, path):
        self.version = version
        self.path = path
        
        if (not self.path) or (not os.path.isdir(self.path)):
            raise Exception('ERROR: could not find path to WinCC OA')

        self.binPath = os.path.join(self.path, "bin")
        if not os.path.isdir(self.binPath):
            raise Exception('ERROR: could not find the bin directory in WinCC OA path')

        self.configPath = os.path.join(self.path, "config")
        if not os.path.isdir(self.configPath):
            raise Exception('ERROR: could not find the config directory in WinCC OA path')
        
    def runManager(self, manager, *args, **kwargs):
        # Fork the environment variables
        if 'env' in kwargs:
            env = kwargs['env'].copy()
            del kwargs['env']
        else:
            env = os.environ.copy()
            
        if platform.system() == 'Linux':
            env = self._setLinuxEnvironment(env)
        
        # Get full path to the manager binary
        manExec = os.path.join(self.binPath, self._getManagerExecName(manager))
        
        execArgs = [manExec]
        if ('console' in args) or ('projAdmin' in args): # Add the default config.pa to set language to english
            if '+config' not in kwargs:
                kwargs['+config'] = []
            kwargs['+config'].append(os.path.join(self.configPath, 'config.pa'))
        
        # Process args given in parameters
        for v in args:
            modifier = ''
            if v in ['console', 'projAdmin', 'n', 'log', '+stderr']:
                if not v.startswith('+'):
                    modifier = '-'
            execArgs.append(modifier + v)
            
        for k, vi in kwargs.iteritems():
            if k in ['config', '+config', 'proj', 'm', 'p']:
                modifier = '' if k.startswith('+') else '-'
                if type(vi) != type([]):
                    vi = [vi]
                for v in vi:
                    execArgs.append(modifier + k)
                    execArgs.append(v)

        # If windows, replace the slashes by anti-slashes
        if platform.system() == 'Windows':
            execArgs = [string.replace(arg, "/", "\\") for arg in execArgs] # ie a single backslash!
            
        # Finally run the manager
        subprocess.check_call(execArgs, env=env)
        
    def runManagerInProject(self, manager, project, *args, **kwargs):
        kwargs['proj'] = project
        self.runManager(manager, *args, **kwargs)
        
    def runManagerInFakeProject(self, manager, preRunHook, *args, **kwargs):
        if 'tempProjectDir' in kwargs:
            tempProjectDir = kwargs['tempProjectDir']
        else:
            tempProjectDir = tempfile.mkdtemp(prefix='WinCCOA-')
        
        os.makedirs(os.path.join(tempProjectDir, 'config'))
        os.makedirs(os.path.join(tempProjectDir, 'log'))
        
        configFileContent = """[general]
pvss_path = "%(pvssPath)s"
proj_path = "%(projPath)s"
proj_version = "%(projVersion)s"
langs = "en_US.%(encoding)s"
userName = "root"
password = ""
"""
        configFileArgs = {
            'pvssPath': self.path,
            'projPath': tempProjectDir,
            'projVersion': self.version,
            'encoding': ('utf8' if self.version != '3.8' else 'iso88591')
        }
        
        configFile = os.path.join(tempProjectDir, 'config', 'config')
        with open(configFile, "w") as fileOut:
            fileOut.write(configFileContent % configFileArgs)
            
        args = tuple('n') + args
        kwargs['config'] = os.path.join(self.configPath, 'config')
        if '+config' not in kwargs:
            kwargs['+config'] = []
        kwargs['+config'].append(configFile)
        
        preRunHook(tempProjectDir)
        
        self.runManager(manager, *args, **kwargs)
        
        return tempProjectDir
    
    def runStandaloneCTL(self, ctlFiles, **kwargs):
        if type(ctlFiles) != type([]):
            ctlFiles = [ctlFiles]
            
        if not ctlFiles:
            return ""
            
        def preRunHook(projectDir):
            scriptDir = os.path.join(projectDir, 'scripts')
            os.makedirs(scriptDir)
            for ctlFile in ctlFiles:
                shutil.copy(ctlFile, scriptDir)
                
        return self.runManagerInFakeProject('ctrl', preRunHook, ctlFiles[0], **kwargs)
    
    def startPA(self):
        self.runManager('ui', 'projAdmin')
    
    def startConsole(self):
        self.runManager('ui', 'console')
        
    def _getManagerExecName(self, manager):
        prefix = 'PVSS00'
        manager = manager.lower()
        if self.version != '3.8':
            if manager in ['data', 'event', 'dist', 'sim', 'redu', 'split', 'proxy', 'databg', 'pmon']:
                prefix = 'WCCIL'
            else:
                prefix = 'WCCOA'
        return prefix + manager

    def _setLinuxEnvironment(self, env):
        if 'PATH' in env:
            if self.binPath not in env['PATH']:
                env['PATH'] = self.binPath + ':' + env['PATH']
        else:
            env['PATH'] = self.binPath

        if 'LD_LIBRARY_PATH' in env:
            if self.binPath not in env['LD_LIBRARY_PATH']:
                env['LD_LIBRARY_PATH'] = self.binPath + ':' + env['LD_LIBRARY_PATH']
        else:
            env['LD_LIBRARY_PATH'] = self.binPath
            
        return env
    
    def __str__(self):
        return self.version
    
    def __repr__(self):
        return 'WinCCOA("%s", "%s")' % (self.version, self.path)

        
    
#------------------------------------------------------------------------------------
        
def getKnownInstalledVersionsList():
    '''Get a list of version strings, eg ['3.11'] Windows or Linux.'''
    return [key for key in getAvailableVersions().keys()]

# ------------------------------------------------------------ 
def getAvailableVersions():
    '''Get a dictionary { pvss_version : pvss_path }'''
    if platform.system() == 'Windows':
        return _findWinCCOAOnWindows()
        
    elif platform.system() == 'Linux':
        return _readLinuxRegistry()
        
    else:
        raise Exception('ERROR: Platform not supported: ' + platform.system())        
    
def getInstances():
    instances = []
    for version, path in getAvailableVersions().iteritems():
        instances.append(WinCCOA(version, path))
        
    return instances

def getDefaultInstance(): # For backwards compatibility
    return getLatestWinCCOA()

def getLatestWinCCOA():
    versions = getAvailableVersions()
    versionsSorted = sorted(versions.keys(), key=lambda s: map(int, s.split('.')))
    lastOne = versionsSorted[-1]
    pathOfLastOne = versions[lastOne]
    return WinCCOA(lastOne, pathOfLastOne)
    
def _findWinCCOAOnWindows():
    versions = {}
    for path in PVSS_WIN_PATHS:
        if os.path.isdir(path) :
            for dir_name in os.listdir(path):
                try:
                    v = float(dir_name) # Important to trigger an exception when dir_name is not a number
                    if v == 3.8:
                        versions[dir_name] = os.path.join(path, dir_name)
                    else:
                        versions[dir_name] = os.path.join(path, dir_name) + '\\'
                except ValueError: # Ignore non-numeric folder names
                    pass
    return versions
    
def _readLinuxRegistry():
    registry = '/etc/opt/pvss/pvssInst.conf'

    if ('PVSS_II_ROOT' in os.environ) and (os.environ['PVSS_II_ROOT']):
        registry = os.environ['PVSS_II_ROOT'] + '/pvssInst.conf'
        print 'PVSS_II_ROOT is set: using registry file ' + registry

    if not os.path.isfile(registry):
        raise Exception('ERROR: env PVSS_II_ROOT set but ' + registry + ' not found! Aborted.')

    # Replace the function GetPv2Path() in shell scripts...
    with open(registry, 'r') as reg:
        # \[Software/ETM/PVSS II/(?!Configs)(.*)\]\n => Matches sections like '[Software\ETM\PVSS II\3.6]'. Avoid getting 'Configs' with (?!...). (.*) extracts the version as the first group.
        # (?:.*\n)*? => Skip lines we are not interested in. (?:...) avoid marking (.*\n) as a group. (...)*? is the non-greedy version of (...)*. '.' is by default configured to not match new lines.
        # InstallationDir *= *"(.*)"\n => Matches the InstallationDir line. ' *= *' is to match the equal notation with whatever number of spaces around. '"(.*)"' extracts the installation path as the second group.
        # string.replace(reg.read(), "\\", "/") => convert the back-slashes into slashes (this registry file is made like that).
        # findall will then give us a list of tuples (pvss version, installation path).
        versions = re.findall(r'\[Software/ETM/PVSS II/(?!Configs)(.*)\]\n(?:.*\n)*?InstallationDir *= *"(.*)"\n', string.replace(reg.read(), "\\", "/"))

    return dict(versions) # Convert to dictionary

