import os
import re
import socket
import sys
import time
from collections import namedtuple

import PVSSBootstrapper as PBS

class PmonException(Exception):
    pass

class WinCCOAPmon(object):
    def __init__(self, project, user='', password=''):
        self.project = project
        self.user = user
        self.password = password
        self.socket = None
        
    def _get_pmon_port(self):
        config_file = os.path.join(self.project.projectPath, 'config', 'config')
        with open(config_file, 'r') as cfg:
            configs = cfg.readlines()
            
        OPTCRE = re.compile(
            r'(?P<option>[^:=\s][^:=]*)'          # very permissive!
            r'\s*(?P<vi>[:=])\s*'                 # any number of space/tab,
                                                  # followed by separator
                                                  # (either : or =), followed
                                                  # by any # space/tab
            r'(?P<value>.*)$'                     # everything up to eol
        )
        
        pmonPort = 4999
        for line in configs:
            mo = OPTCRE.match(line)
            if mo:
                optname, vi, optval = mo.group('option', 'vi', 'value')
                optname = optname.rstrip()
                if optname == 'pmonPort':
                    pmonPort = int(optval)
        
        return pmonPort
        
    def spawn_pmon(self, **kwargs):
         PBS.startProject(self.project.projectName, onlyPmon=True, noConsole=True, noLogViewer=True, **kwargs)
        
    def _connect(self, host='localhost'):
        port = self._get_pmon_port()
        self.socket = None
        for res in socket.getaddrinfo(host, port, socket.AF_UNSPEC, socket.SOCK_STREAM):
            af, socket_type, proto, canonname, sa = res
            try:
                self.socket = socket.socket(af, socket_type, proto)
            except socket.error:
                exc_info = sys.exc_info()
                self.socket = None
                continue
            
            try:
                self.socket.connect(sa)
            except socket.error:
                exc_info = sys.exc_info()
                self.socket.close()
                self.socket = None
                continue
            break
            
        if self.socket is None:
            raise exc_info[1], None, exc_info[2]
            
    def _send_command(self, command, sub_command=None, arguments=None):
        msg = '{user}#{password}#{command}:'
        msg = msg.format(user=self.user, password=self.password, command=command)
        if sub_command:
            msg += sub_command
        if arguments is not None:
            if hasattr(arguments, '__iter__'):
                msg += ' ' + ' '.join(map(lambda a: str(a), arguments))
            else:
                msg += ' ' + str(arguments)
                   
        #print 'Sending', msg
        msg += '\n'
            
        if self.socket is None:
            self._connect()
            
        self.socket.sendall(msg)
        return self.socket.recv(4096)
    
    def start_all(self):
        ret = self._send_command('START_ALL')
        if ret != 'OK':
            raise PmonException(ret)
    
    def restart_all(self):
        ret = self._send_command('RESTART_ALL')
        if ret != 'OK':
            raise PmonException(ret)
            
    def stop_all(self):
        ret = self._send_command('STOP_ALL')
        if ret != 'OK':
            raise PmonException(ret)
            
    start_modes = ['manual', 'once', 'always']
    states = ['stopped', 'init', 'running', 'blocked']
    ManagerProperties = namedtuple('ManagerProperties', 'i name start_mode sec_kill restart_count reset_min command_line')
    ManagerStatus = namedtuple('ManagerStatus', 'i state pid start_mode start_time man_num')
    Manager = namedtuple('Manager', 'i name command_line pid state man_num start_time start_mode sec_kill restart_count reset_min')
    PmonStatus = namedtuple('PmonStatus', 'mode_num mode emergency_active demo_active')
            
    def manager_list(self):
        raw_list = self._send_command('MGRLIST', 'LIST')
        if not raw_list.startswith('LIST:'):
            raise Exception('Pmon returned an unexpected answer: %r' % raw_list)
        while not raw_list.endswith('\n;\n'): # Somehow, we don't always receive the full list in one go
            raw_list += self.socket.recv(4096)
            
        raw_list = raw_list.split('\n')
        mgr_list = []
        i = 0
        for line in raw_list:
            if line == ';':
                break
            if line.startswith('LIST:'):
                continue
                
            m = self.ManagerProperties._make([i,] + line.split(';'))
            m = m._replace(
                start_mode = self.start_modes[int(m.start_mode)],
                sec_kill = int(m.sec_kill),
                restart_count = int(m.restart_count),
                reset_min = int(m.reset_min)
            )
            mgr_list.append(m)
            i += 1
            
        return mgr_list
        
    def managers_status(self):
        raw_list = self._send_command('MGRLIST', 'STATI')
        if not raw_list.startswith('LIST:'):
            raise Exception('Pmon returned an unexpected answer: %r' % raw_list)
        while not raw_list.endswith('\n;\n'): # Somehow, we don't always receive the full list in one go
            raw_list += self.socket.recv(4096)
            
        raw_list = raw_list.split('\n')
        status_list = []
        max_lines = int(raw_list[0][len('LIST:'):]) # Read XXX in 'LIST:XXX'
        i = 0
        for line in raw_list:
            if (i >= max_lines) or (line == ';'):
                break
            if line.startswith('LIST:'):
                continue
                
            m = self.ManagerStatus._make([i,] + line.split(';'))
            m = m._replace(
                state = self.states[int(m.state)],
                pid = int(m.pid),
                start_mode = self.start_modes[int(m.start_mode)],
                man_num = int(m.man_num)
            )
            status_list.append(m)
            i += 1
            
        return status_list
        
    def manager_list_status(self):
        mgr_list = self.manager_list()
        statuses = self.managers_status()
        
        mgr_status_list = []
        for manager, status in zip(mgr_list, statuses):
            d = manager._asdict()
            d.update(status._asdict())
            mgr_status_list.append(self.Manager(**d))
            
        return mgr_status_list
        
    def pmon_status(self):
        raw_list = self._send_command('MGRLIST', 'STATI')
        if not raw_list.startswith('LIST:'):
            raise Exception('Pmon returned an unexpected answer: %r' % raw_list)
        while not raw_list.endswith('\n;\n'):
            raw_list += self.socket.recv(4096)
            
        raw_list = raw_list.split('\n')
        max_lines = int(raw_list[0][len('LIST:'):]) # Read XXX in 'LIST:XXX'
        status = self.PmonStatus._make(raw_list[max_lines+1].split(' '))
        status = status._replace(
            mode_num = int(status.mode_num),
            emergency_active = bool(int(status.emergency_active)),
            demo_active = bool(int(status.demo_active))
        )
        return status
        
    def _query_managers(self, query, prop_filter=None):
        try:
            return [int(query)]
        except TypeError: # Not an int or string
            if hasattr(manager, 'i'):
                return [query.i]
            if hasattr(query, 'name'):
                string_to_match = query.name
                if hasattr(query, 'command_line'):
                    string_to_match = ' '.join((string_to_match, query.command_line))
            else:
                string_to_match = str(query)
        except ValueError: # String not representing an int
            string_to_match = query
            
        if not string_to_match.startswith('^'):
            string_to_match = '^' + string_to_match
        if not string_to_match.endswith('$'):
            string_to_match += '$'
            
        matches = []
        for m in self.manager_list_status():
            m_cmd_line = ' '.join((m.name, m.command_line)).rstrip()
            if re.match(string_to_match, m_cmd_line):
                if prop_filter and not prop_filter(m):
                    continue
                matches.append(m)
                
        if not matches:
            raise ValueError('Could not find any manager for query %r' % query)
        else:
            return matches
            
    def start_manager(self, query, prop_filter=None, ignore_exception=False, wait=False):
        for manager in self._query_managers(query, prop_filter):
            ret = self._send_command('SINGLE_MGR', 'START', manager.i)
            if not ignore_exception and (ret != 'OK'):
                raise PmonException(ret)
                
            if wait:
                while self.managers_status()[manager.i].state != 'running':
                    time.sleep(0.25)
            
    def stop_manager(self, query, prop_filter=None, ignore_exception=False):
        for manager in self._query_managers(query, prop_filter):
            ret = self._send_command('SINGLE_MGR', 'STOP', manager.i)
            if not ignore_exception and (ret != 'OK'):
                raise PmonException(ret)
            
    def kill_manager(self, query, prop_filter=None, ignore_exception=False):
        for manager in self._query_managers(query, prop_filter):
            ret = self._send_command('SINGLE_MGR', 'KILL', manager.i)
            if not ignore_exception and (ret != 'OK'):
                raise PmonException(ret)
            
    def delete_manager(self, query, prop_filter=None, ignore_exception=False):
        for manager in self._query_managers(query, prop_filter):
            ret = self._send_command('SINGLE_MGR', 'DEL', manager.i)
            if not ignore_exception and (ret != 'OK'):
                raise PmonException(ret)
            
    def get_manager_properties(self, query, prop_filter=None, ignore_exception=False):
        for manager in self._query_managers(query, prop_filter):
            props = self._send_command('SINGLE_MGR', 'PROP_GET', manager.i)
            
            # command_line can contains spaces, rebuild the prop
            props_list = props.split(' ')
            props_list = props_list[:4] + [' '.join(props_list[4:]).rstrip()]
            
            props = self.ManagerProperties._make([manager.i, manager.name] + props_list)
            props = props._replace(
                sec_kill = int(props.sec_kill),
                restart_count = int(props.restart_count),
                reset_min = int(props.reset_min)
            )
            
            yield props
            
    def set_manager_properties(self, manager):
        props = manager._asdict()
        
        fields = ['i', 'start_mode', 'sec_kill', 'restart_count', 'reset_min', 'command_line']
        props_values = [] # Ensure order of params by building manually the list
        for field in fields:
            props_values.append(props[field])
        
        ret = self._send_command('SINGLE_MGR', 'PROP_PUT', props_values)
        if ret != 'OK':
            raise PmonException(ret)
            
    def update_manager_properties(self, query, prop_filter=None, ignore_exception=False, **props):
        modified_managers = []
        for manager in self._query_managers(query, prop_filter):
            self.set_manager_properties(manager._replace(**props))
            modified_managers.append(manager)
        return modified_managers
            
    def project_name(self):
        return self._send_command('PROJECT')
        
