#!/bin/sh

###
# The aim of this script is to setup the execution environment
# in such a way we can start a project-independant PVSS CTRL script.
# Based on the startPA script from the PVSS bundle.
###

###### Script variables:
ctrlConfig="config/config.PVSSBootstrapper"
ctrlScript="libs/PVSSBootstrapper/PVSSBootstrapper.ctl"
###### end

###### PVSS variables:
CTRL=PVSS00ctrl
REGDIR_STD=/etc/opt/pvss
CONFIG_STD=pvssInst.conf
###### end brandable

#------------ usage ----------------------

usage() {
  cat << EOF
Usage: $0 options

This script run the PVSSBootstrapper.

OPTIONS:
 -c | --config-file FILE                       Use an external config file. It will have priority on the command line paramaters.
 --config-source (ConfigFile|ConfigFile)       Define if the main configuration is coming from a config file or from fwConfigurationDBSystemInformation.
 -g | --debug                                  Activate debug flag.
 
 --db TNS_NAME                                 Database TNS name for fwConfigurationDBSystemInformation.
 --db-user USERNAME                            Database user name for fwConfigurationDBSystemInformation.
 --db-encrypted-password ENC_PASSWORD          Database password for fwConfigurationDBSystemInformation. Password is encrypted by the PVSS CtrlRDBAccess rdbencrypt() function.
 
 --procedure (ProjectStarter|ProjectCreation)  Set the procedure that will be followed.
 --execution-mode (OneShot|Agent)              Set the execution mode of the tool (default: OneShot).
 -a | --agent                                  Start in agent mode. Automatically set the fwConfigurationDBSystemInformation config source.
 --agent-sleep DELAY                           Specify the refresh delay of the agent in seconds (default: 5).
 
 --project-install PATH                        Create a project in the given directory. Automatically set the ProjectCreation procedure.
 --project-name NAME                           Give the name of the project to create.
 --project-system-number N                     Set the system number of the project to create (default: 1).
 --project-delete-if-exists (true|false)       If a similar project exists, should it be deleted first? (default: false)
 --project-pmon-port N                         Set the port for pmon of the project to create (default: 0).
 --project-data-port N                         Set the port for the data manager of the project to create (default: 0).
 --project-event-port N                        Set the port for the event manager of the project to create (default: 0).
 --project-dist-port N                         Set the port for the distributed manager of the project to create (default: 0).
 --fwInstallation-zip-path PATH                If set, tells to install the fwInstallation tool in the created project.
 --component-install-dir PATH                  Directory in which the installation tool will install the components.
 
 --start-project PATH                          Only start the given project. Automatically set the ProjectStarter procedure.
 --started-with-manager MAN                    Tell which manager should be running to determine the project is running (eg. PVSS00ui).
 --started-with-options OPTIONS                Options of the manager that help filtering out the wrong managers (eg. "-m gedi").
 --open-console (true|false)                   Open a console when starting a project (default: true).
 --open-log-viewer (true|false)                Open a log viewer when starting a project (default: true).
 
 -h | --help                                   This message.
 
EOF
}

#------------ globals --------------------
RegProj=$REGDIR_STD
RegFile=$CONFIG_STD

REGISTRY=$RegProj/$RegFile
REG_TEST=$PVSS_II_ROOT/$RegFile

GetPv2Path() # [<version>]
{
  vers="$1"

  cfg=$( set -- $REGISTRY ; echo $1 )
  grep -e "^\[Software" -e "^InstallationDir *=" $cfg | sed 's@\\@/@g' | ( 
    while read line; do
      IFS="/[]"
      set -- $line
      IFS="   "

      # [Software\ETM\PVSS II\3.0]
      # [Software\ETM\PVSS II\Configs\projName]
      if [ "$2" = "Software" -a "$3" = "ETM" -a "$4" = "PVSS II" ]; then
        if [ \( -n "$vers" -a "$5" = "$vers" \) -o \( -z "$vers" -a "$5" != "Configs" \) ]; then
          read line;
          IFS="   ="
          set $line
          #    v-TAB!
          IFS="	 "
          if [ "$1" = "InstallationDir" ]; then
            echo $line | cut -d\" -f2
            break
          fi
        fi
      fi
    done
  )
}

if [ "$PVSS_II_ROOT" != "" ]; then
  if [ -f "$REG_TEST" ]; then
    REGISTRY=$REG_TEST
    echo "PVSS_II_ROOT is set: using registry file $REGISTRY."
  else
    echo "ERROR: env PVSS_II_ROOT set but $REG_TEST not found! Aborted."
    exit 3
  fi
fi

pv2Path=$( GetPv2Path "3.8" )
if [ -z "${pv2Path}" -o ! -d "${pv2Path}" ]; then
  echo "Installation ${pv2Path} not found, check file $REGISTRY for proper installation ! Aborted."
  exit 2
fi

pv2Bin=$pv2Path/bin
export PATH="${pv2Bin}:$PATH"
if [ ! -d "${pv2Bin}" ]; then
  echo "Installation directory ${pv2Bin} not found ! Aborted."
  exit 3
fi

CTRL=$pv2Bin/$CTRL
if [ ! -x "$CTRL" ]; then
  echo "Executable $CTRL not found ! Aborted."
  exit 4
fi

if [ $( uname ) = "Linux" -o $( uname ) = "SunOS" ]
then
  export LD_LIBRARY_PATH="$pv2Bin:${LD_LIBRARY_PATH}:/usr/local/lib"
fi

pv2Config=$pv2Path/config
if [ ! -d "${pv2Config}" ]; then
  echo "Installation directory ${pv2Config} not found ! Aborted."
  exit 3
fi

currentPath=`pwd`

# Make a custom config file with proper paths
cp $ctrlConfig.model $ctrlConfig
sed -i "s,#PVSS_PATH#,$pv2Path," $ctrlConfig
sed -i "s,#PROJ_PATH#,$currentPath," $ctrlConfig

while [ $# -gt 0 ]; do
  case $1 in
#    -t) # Template file
#      templateFile=$2
#      shift
#      ;;
#    -o) # Output file
#      outputFile=$2
#      shift
#      ;;

    -c | --config-file) # Config file (absolute path)
      configFile=`readlink -f "$2"`
      sed -i "s,#CONFIG_FILE#,$configFile," $ctrlConfig
      shift
      ;;
    --config-source)
      sed -i "s,#CONFIG_SOURCE#,$2," $ctrlConfig
      shift
      ;;
    -g | --debug) # debug flag
      sed -i "s,#DEBUG#,true," $ctrlConfig
      ;;
      
    --db) # Database name
      sed -i "s,#CONFIG_DB#,$2," $ctrlConfig
      shift
      ;;
    --db-user) # Database user
      sed -i "s,#CONFIG_DB_USER#,$2," $ctrlConfig
      shift
      ;;
    --db-encrypted-password) # Database password
      sed -i "s,#CONFIG_DB_ENC_PWD#,$2," $ctrlConfig
      shift
      ;;
      
    --procedure)
      sed -i "s,#PROCEDURE#,$2," $ctrlConfig
      shift
      ;;
    --execution-mode)
      sed -i "s,#EXEC_MODE#,$2," $ctrlConfig
      shift
      ;;
    -a | --agent) # Agent mode
      sed -i "s,#CONFIG_SOURCE#,ConfDBSysInfo," $ctrlConfig
      sed -i "s,#EXEC_MODE#,Agent," $ctrlConfig
      ;;
    --agent-sleep) # Agent sleep
      sed -i "s,#AGENT_SLEEP#,$2," $ctrlConfig
      shift
      ;;
      
    --project-install)
      sed -i "s,#CONFIG_SOURCE#,ConfigFile," $ctrlConfig
      sed -i "s,#PROCEDURE#,ProjectCreation," $ctrlConfig
      sed -i "s,#INSTALL_DIR#,$2," $ctrlConfig
      shift
      ;;
    --project-name)
      sed -i "s,#PROJECT_NAME#,$2," $ctrlConfig
      shift
      ;;
    --project-system-number)
      sed -i "s,#SYS_NUM#,$2," $ctrlConfig
      shift
      ;;
    --project-delete-if-exists)
      sed -i "s,#DELETE_FIRST_IF_EXISTS#,$2," $ctrlConfig
      shift
      ;;
    --project-pmon-port)
      sed -i "s,#PMON_PORT#,$2," $ctrlConfig
      shift
      ;;
    --project-data-port)
      sed -i "s,#DATA_PORT#,$2," $ctrlConfig
      shift
      ;;
    --project-event-port)
      sed -i "s,#EVENT_PORT#,$2," $ctrlConfig
      shift
      ;;
    --project-dist-port)
      sed -i "s,#DIST_PORT#,$2," $ctrlConfig
      shift
      ;;
      
    --fwInstallation-zip-path)
      sed -i "s,#FWINST_ZIP_PATH#,$2," $ctrlConfig
      shift
      ;;
      
    --component-install-dir)
      sed -i "s,#COMP_INSTALL_DIR#,$2," $ctrlConfig
      shift
      ;;
      
    --start-project) # ProjectStarter
      sed -i "s,#CONFIG_SOURCE#,ConfigFile," $ctrlConfig
      sed -i "s,#PROCEDURE#,ProjectStarter," $ctrlConfig
      sed -i "s,#START_PROJECT_PATH#,$2," $ctrlConfig
      shift
      ;;
    --started-with-manager) # isProjectStarted condition manager
      sed -i "s,#IS_PROJ_STARTED_CONDITION_TYPE#,PVSS manager running," $ctrlConfig
      sed -i "s,#IS_PROJ_STARTED_CONDITION_MANAGER#,$2," $ctrlConfig
      shift
      ;;
    --started-with-options) # isProjectStarted condition options
      sed -i "s,#IS_PROJ_STARTED_CONDITION_OPTIONS#,$2," $ctrlConfig
      shift
      ;;
    --open-console)
      sed -i "s,#OPEN_CONSOLE#,$2," $ctrlConfig
      shift
      ;;
    --open-log-viewer)
      sed -i "s,#OPEN_LOG_VIEWER#,$2," $ctrlConfig
      shift
      ;;
      
    -h | --help) # isProjectStarted condition options
       usage
       exit
       ;;
    *) 
       echo "$1: unknown option"
       usage
       exit
  esac
  shift
done

# Clean the config file
sed -i "/#.*#/d" $ctrlConfig # Remove lines where there is still replacement markers in it
sed -i "s,#.*,," $ctrlConfig # Remove comments
sed -i "/^$/d" $ctrlConfig # Remove empty lines

MAN=$CTRL
startOpts="-config $pv2Config/config +config `pwd`/$ctrlConfig -n $ctrlScript -log +stderr"

$MAN $startOpts


