import os
import shutil
from PVSSBootstrapper import PVSSBootstrapper
from PVSSBootstrapper._python_compat import *
from PVSSBootstrapperTests import PBSTestCase
import PVSSManagers

# unittest with extended functionalities is from Python >= 2.7
# Load compatible package (install it before) unittest2
try:
  import unittest2 as unittest
except ImportError:
  import unittest

projectName = "PBSTest_createProject"

class CreateProjectTests(PBSTestCase):
    
  @classmethod
  def setUpClass(cls):
    PBSTestCase.setUpClass()
    
    PVSSBootstrapper.stopAllProjects()
    # In case the project did not get deleted, clean up, because we are not using the overwrite option in these tests
    if PVSSBootstrapper.isProjectRegistered(projectName):
      PVSSBootstrapper.deleteProject(projectName)
    else:
      if os.path.exists(os.path.join(cls.sandbox, projectName)):
        shutil.rmtree(os.path.join(cls.sandbox, projectName))

  def setUp(self):
    PBSTestCase.setUp(self)
    PVSSBootstrapper.stopAllProjects()
  
  def tearDown(self):
    PBSTestCase.tearDown(self)
    PVSSBootstrapper.stopProject(projectName, warningLevel=0)
    PVSSBootstrapper.deleteProject(projectName)
    
  def test_installPath_relative(self):
    PVSSBootstrapper.createProject(self.sandbox, projectName)
    self.setGediToManual(os.path.join(self.sandbox, projectName))
    
    # Check it is a valid project
    self.assertTrue(PVSSBootstrapper.isProjectRegistered(projectName),  'The project should be registered')
    PVSSBootstrapper.startProject(projectName, noConsole=True, noLogViewer=True)
    self.assertTrue(PVSSBootstrapper.isProjectRunning(projectName),  'The project should have started correctly')
    
  def test_installPath_absolute(self):
    PVSSBootstrapper.createProject(os.path.abspath(self.sandbox), projectName)
    self.setGediToManual(os.path.join(self.sandbox, projectName))
    
    # Check it is a valid project
    self.assertTrue(PVSSBootstrapper.isProjectRegistered(projectName),  'The project should be registered')
    PVSSBootstrapper.startProject(projectName, noConsole=True, noLogViewer=True)
    self.assertTrue(PVSSBootstrapper.isProjectRunning(projectName),  'The project should have started correctly')
    
  def test_overwrite(self):
    PVSSBootstrapper.createProject(self.sandbox, projectName)
    self.setGediToManual(os.path.join(self.sandbox, projectName))
    
    with self.assertRaisesRegexp(PVSSBootstrapper.PVSSBootstrapperException, 'Project is already registered. Use the overwrite option.') as cm:
      PVSSBootstrapper.createProject(self.sandbox, projectName)
    self.assertNotEqual(cm.exception.retInfo['errCode'], 0, 'Second project creation should have failed')
    
    PVSSBootstrapper.createProject(self.sandbox, projectName, overwrite=True)
    
  def test_systemName(self):
    systemName = 'TEST'
    PVSSBootstrapper.createProject(self.sandbox, projectName, systemName=systemName)
    self.setGediToManual(os.path.join(self.sandbox, projectName))
    
    PVSSBootstrapper.startProject(projectName, noConsole=True, noLogViewer=True)
    
    self.execCTRLScript(os.path.join(self.sandbox, projectName), projectName, 'getSystemInfo.ctl')
    
    configFile = os.path.join(self.sandbox, projectName, 'config', 'config')
    kVs = PVSSBootstrapper._readConfigFile(configFile)
    
    self.assertTrue('test_systemName' in kVs, 'Could not detect the system name of the project')
    self.assertEqual(kVs['test_systemName'], systemName, 'The system name of the project is not the expected one')
    
  def test_systemNumber(self):
    systemNumber = 12
    PVSSBootstrapper.createProject(self.sandbox, projectName, systemNumber=systemNumber)
    self.setGediToManual(os.path.join(self.sandbox, projectName))
    
    PVSSBootstrapper.startProject(projectName, noConsole=True, noLogViewer=True)
    
    self.execCTRLScript(os.path.join(self.sandbox, projectName), projectName, 'getSystemInfo.ctl')
    
    configFile = os.path.join(self.sandbox, projectName, 'config', 'config')
    kVs = PVSSBootstrapper._readConfigFile(configFile)
    
    self.assertTrue('test_systemNumber' in kVs, 'Could not detect the system number of the project')
    self.assertEqual(int(kVs['test_systemNumber']), systemNumber, 'The system number of the project is not the expected one')
    
  @staticmethod
  def findListeningPorts(manager):
    for c in manager.get_connections():
      if c.status == 'LISTEN':
        yield c.local_address[1]
    
  def test_pmonPort(self):
    port=5000
    PVSSBootstrapper.createProject(self.sandbox, projectName, pmonPort=port)
    self.setGediToManual(os.path.join(self.sandbox, projectName))
    
    # Check in the config file
    configFile = os.path.join(self.sandbox, projectName, 'config', 'config')
    kVs = PVSSBootstrapper._readConfigFile(configFile)
    
    self.assertTrue('pmonPort' in kVs, 'Entry pmonPort not found in the config file of the project')
    self.assertEqual(int(kVs['pmonPort']), port, 'pmonPort entry in the config file of the project is not the expected one')
    
    # Check pmon is listening on the proper port
    PVSSBootstrapper.startProject(projectName, noConsole=True, noLogViewer=True)
          
    pmon = PVSSManagers.getPmon(projectName)
    self.assertIsNotNone(pmon, 'The pmon of the project is not running')
    self.assertTrue(any((p == port) for p in self.findListeningPorts(pmon)), 'The pmon of the project is not listening on port ' + str(port))
    
  def test_dataPort(self):
    port=5010
    PVSSBootstrapper.createProject(self.sandbox, projectName, dataPort=port)
    self.setGediToManual(os.path.join(self.sandbox, projectName))
    
    # Check in the config file
    configFile = os.path.join(self.sandbox, projectName, 'config', 'config')
    kVs = PVSSBootstrapper._readConfigFile(configFile)
    
    self.assertTrue('dataPort' in kVs, 'Entry dataPort not found in the config file of the project')
    self.assertEqual(int(kVs['dataPort']), port, 'dataPort entry in the config file of the project is not the expected one')
    
    # Check data is listening on the proper port
    PVSSBootstrapper.startProject(projectName, noConsole=True, noLogViewer=True)
          
    data = next(PVSSManagers.projectManagers(projectName, '^(PVSS00|WCCIL)data(.exe)?$'), None)
    self.assertIsNotNone(data, 'The data manager of the project is not running')
    self.assertTrue(any((p == port) for p in self.findListeningPorts(data)), 'The data manager of the project is not listening on port ' + str(port))
    
  def test_distPort(self):
    port=5020
    PVSSBootstrapper.createProject(self.sandbox, projectName, distPort=port)
    self.setGediToManual(os.path.join(self.sandbox, projectName))
    
    # Check in the config file
    configFile = os.path.join(self.sandbox, projectName, 'config', 'config')
    kVs = PVSSBootstrapper._readConfigFile(configFile)
    
    self.assertTrue('distPort' in kVs, 'Entry distPort not found in the config file of the project')
    self.assertEqual(int(kVs['distPort']), port, 'distPort entry in the config file of the project is not the expected one')
    
    # Check dist is listening on the proper port
    PVSSBootstrapper.startProject(projectName, noConsole=True, noLogViewer=True)
          
    dist = next(PVSSManagers.projectManagers(projectName, '^(PVSS00|WCCIL)dist(.exe)?$'), None)
    self.assertIsNotNone(dist, 'The dist manager of the project is not running')
    self.assertTrue(any((p == port) for p in self.findListeningPorts(dist)), 'The dist manager of the project is not listening on port ' + str(port))
    
  def test_eventPort(self):
    port=5030
    PVSSBootstrapper.createProject(self.sandbox, projectName, eventPort=port)
    self.setGediToManual(os.path.join(self.sandbox, projectName))
    
    # Check in the config file
    configFile = os.path.join(self.sandbox, projectName, 'config', 'config')
    kVs = PVSSBootstrapper._readConfigFile(configFile)
    
    self.assertTrue('eventPort' in kVs, 'Entry eventPort not found in the config file of the project')
    self.assertEqual(int(kVs['eventPort']), port, 'eventPort entry in the config file of the project is not the expected one')
    
    # Check event is listening on the proper port
    PVSSBootstrapper.startProject(projectName, noConsole=True, noLogViewer=True)
          
    event = next(PVSSManagers.projectManagers(projectName, '^(PVSS00|WCCIL)event(.exe)?$'), None)
    self.assertIsNotNone(event, 'The event manager of the project is not running')
    self.assertTrue(any((p == port) for p in self.findListeningPorts(event)), 'The event manager of the project is not listening on port ' + str(port))
