import os
import shutil
from PVSSBootstrapper import PVSSBootstrapper
from PVSSBootstrapper._python_compat import *
from PVSSBootstrapperTests import PBSTestCase
import PVSSManagers

# unittest with extended functionalities is from Python >= 2.7
# Load compatible package (install it before) unittest2
try:
  import unittest2 as unittest
except ImportError:
  import unittest

projectName = "PBSTest_registerProject"

class RegisterProjectTests(PBSTestCase):

  @classmethod
  def setUpClass(cls):
    PBSTestCase.setUpClass()
    PVSSBootstrapper.stopAllProjects()
    PVSSBootstrapper.createProject(cls.sandbox, projectName, overwrite=True)
    cls.projectPath = os.path.join(cls.sandbox, projectName)
    cls.setGediToManual(cls.projectPath)

  @classmethod
  def tearDownClass(cls):
    # In case the project did not get re-registered in a failed or invalid case
    if PVSSBootstrapper.isProjectRegistered(projectName):
      PVSSBootstrapper.deleteProject(projectName)
    else:
       if os.path.exists(cls.projectPath):
         shutil.rmtree(cls.projectPath)
        
    PBSTestCase.tearDownClass()

  def setUp(self):
    PBSTestCase.setUp(self)
    PVSSBootstrapper.stopAllProjects()
    
    # In case the project did not get re-registered in a failed or invalid case
    if PVSSBootstrapper.isProjectRegistered(projectName):
      PVSSBootstrapper.unregisterProject(projectName)
  
  def tearDown(self):
    PBSTestCase.tearDown(self)
    PVSSBootstrapper.stopAllProjects()
    
  def test_projectPath_valid(self):
    PVSSBootstrapper.registerProject(self.projectPath)
    
    self.assertTrue(PVSSBootstrapper.isProjectRegistered(projectName))
    
  def test_projectPath_invalid(self):
    with self.assertRaisesRegexp(PVSSBootstrapper.PVSSBootstrapperException, 'PVSSBootstrapper_registerProject\(\) -> paRegProj\(\) returned an error: -1. Registration incomplete') as cm:
      PVSSBootstrapper.registerProject('blablabla')
    self.assertNotEqual(cm.exception.retInfo['errCode'], 0, 'Registering project blablabla should have failed')
    
  def test_pmonPort(self):
    port = 8000 # Don't use 6000, it's x11 port, used in virtual box images of SLC5
    PVSSBootstrapper.registerProject(self.projectPath, pmonPort=port)
    
    self.assertTrue(PVSSBootstrapper.isProjectRegistered(projectName))
    
    # Check in the config file
    configFile = os.path.join(self.projectPath, 'config', 'config')
    kVs = PVSSBootstrapper._readConfigFile(configFile)
    self.assertTrue('pmonPort' in kVs, 'Entry pmonPort not found in the config file of the project')
    self.assertEqual(int(kVs['pmonPort']), port, 'pmonPort entry in the config file of the project is not the expected one')
    
    # Check pmon is listening on the proper port
    PVSSBootstrapper.startProject(projectName, noConsole=True, noLogViewer=True)
    
    def findPmonPort(pmon):
      for c in pmon.get_connections():
        if c.status == 'LISTEN':
          yield c.local_address[1]
          
    pmon = PVSSManagers.getPmon(projectName)
    self.assertIsNotNone(pmon, 'The pmon of the project is not running')
    self.assertTrue(any((p == port) for p in findPmonPort(pmon)), 'The pmon of the project is not listening on port ' + str(port))
