import os
import shutil
import time
from PVSSBootstrapper import PVSSBootstrapper
from PVSSBootstrapperTests import PBSTestCase
import PVSSManagers

# unittest with extended functionalities is from Python >= 2.7
# Load compatible package (install it before) unittest2
try:
  import unittest2 as unittest
except ImportError:
  import unittest

projectName = "PBSTest_startProject"

class StartProjectTests(PBSTestCase):

  @classmethod
  def setUpClass(cls):
    PBSTestCase.setUpClass()
    PVSSBootstrapper.stopAllProjects()
    PVSSBootstrapper.createProject(cls.sandbox, projectName, overwrite=True)
    cls.projectPath = os.path.join(cls.sandbox, projectName)
    cls.setGediToManual(cls.projectPath)

  @classmethod
  def tearDownClass(cls):
    if PVSSBootstrapper.isProjectRegistered(projectName):
      PVSSBootstrapper.deleteProject(projectName)
      pass
    else:
       if os.path.exists(cls.projectPath):
         shutil.rmtree(cls.projectPath)
        
    PBSTestCase.tearDownClass()

  def setUp(self):
    PBSTestCase.setUp(self)
    PVSSBootstrapper.stopAllProjects()
    PVSSBootstrapper.registerProject(self.projectPath, warningLevel=0)
  
  def tearDown(self):
    PBSTestCase.tearDown(self)
    PVSSBootstrapper.stopAllProjects()
    
    # On windows the console process does not quit automatically when the project is stopped.
    # Use a while loop because in test_restart two consoles are started.
    while PVSSManagers.getConsole():
        PVSSManagers.getConsole().kill()
    
  def test_projectName_valid(self):
    PVSSBootstrapper.startProject(projectName)
    self.assertTrue(PVSSBootstrapper.isProjectRunning(projectName), 'Project does not appear to be running')
    
  def test_projectName_invalid(self):
    with self.assertRaisesRegexp(PVSSBootstrapper.PVSSBootstrapperException, 'PVSSBootstrapper_startProject\(\) -> Unable to find the project blablabla in the list of registered projects') as cm:
      PVSSBootstrapper.startProject('blablabla')
    self.assertNotEqual(cm.exception.retInfo['errCode'], 0, 'Starting the project blablabla should have failed')
    
  def test_projectPath_valid(self):
    PVSSBootstrapper.startProject(self.projectPath)
    self.assertTrue(PVSSBootstrapper.isProjectRunning(projectName), 'Project does not appear to be running')
    
  def test_projectPath_invalid(self):
    with self.assertRaisesRegexp(PVSSBootstrapper.PVSSBootstrapperException, 'PVSSBootstrapper_registerProject\(\) -> paRegProj\(\) returned an error: -1. Registration incomplete') as cm:
      PVSSBootstrapper.startProject(self.sandbox) # Need an existing directory to pass the filter in the Python script.
    self.assertNotEqual(cm.exception.retInfo['errCode'], 0, 'Starting from an invalid project path should have failed')
    
  def test_restart(self):
    tStart = time.time()
    PVSSBootstrapper.startProject(projectName)
    tStop = time.time()
    tStartUp = tStop - tStart

    tStart = time.time()
    PVSSBootstrapper.startProject(projectName, restart=True)
    tStop = time.time()
    self.assertLess(tStartUp, tStop-tStart, 'It does not appear the project restarted')
    self.assertTrue(PVSSBootstrapper.isProjectRunning(projectName), 'Project does not appear to be running')
    
  def test_dontRestart(self):
    PVSSBootstrapper.startProject(projectName)
    
    with self.assertRaisesRegexp(PVSSBootstrapper.PVSSBootstrapperException, 'PVSSBootstrapper_startProject\(\) -> Project ' + projectName + ' already running') as cm:
      PVSSBootstrapper.startProject(projectName, warningLevel=2)
    self.assertNotEqual(cm.exception.retInfo['errCode'], 0, 'Restarting the project should have failed')
  
  def test_autoConsole(self):
    PVSSBootstrapper.startProject(projectName)
    self.assertTrue(PVSSBootstrapper.isProjectRunning(projectName), 'Project does not appear to be running')
    
    console = PVSSManagers.getConsole()
    self.assertTrue(console != None, 'The console is not running')
    self.assertTrue(console.is_running(), 'The console is not running')
  
  def test_noConsole(self):
    PVSSBootstrapper.startProject(projectName, noConsole=True)
    self.assertTrue(PVSSBootstrapper.isProjectRunning(projectName), 'Project does not appear to be running')
    
    console = PVSSManagers.getConsole()
    self.assertTrue(console == None, 'The console is running')

  def test_autoLogViewer(self):
    PVSSBootstrapper.startProject(projectName)
    self.assertTrue(PVSSBootstrapper.isProjectRunning(projectName), 'Project does not appear to be running')
    
    logViewer = PVSSManagers.getLogViewer(projectName)
    self.assertTrue(logViewer != None, 'The log viewer of the project is not running')
    self.assertTrue(logViewer.is_running(), 'The log viewer of the project is not running')
  
  #@unittest.expectedFailure
  #def test_noLogViewer(self):
  #  PVSSBootstrapper.startProject(projectName, noLogViewer=True)
  #  self.assertTrue(PVSSBootstrapper.isProjectRunning(projectName), 'Project does not appear to be running')
  #  
  #  logViewer = PVSSManagers.getLogViewer(projectName)
  #  self.assertTrue(logViewer == None, 'The log viewer of the project is running')

  def test_autoRegister(self):
    PVSSBootstrapper.unregisterProject(projectName)
    
    PVSSBootstrapper.startProject(self.projectPath)
    self.assertTrue(PVSSBootstrapper.isProjectRunning(projectName), 'Project does not appear to be running')
    self.assertTrue(PVSSBootstrapper.isProjectRegistered(projectName), 'Project does not appear to be registered')
    
  def test_dontRegister(self):
    PVSSBootstrapper.unregisterProject(projectName)
    
    with self.assertRaisesRegexp(PVSSBootstrapper.PVSSBootstrapperException, 'PVSSBootstrapper_startProject\(\) -> Unable to find the project ' + projectName + ' in the list of registered projects') as cm:
      PVSSBootstrapper.startProject(self.projectPath, dontRegister=True)
    self.assertNotEqual(cm.exception.retInfo['errCode'], 0, 'Starting the unregistered project by its path should have failed')
    
    with self.assertRaisesRegexp(PVSSBootstrapper.PVSSBootstrapperException, 'PVSSBootstrapper_startProject\(\) -> Unable to find the project ' + projectName + ' in the list of registered projects') as cm:
      PVSSBootstrapper.startProject(projectName)
    self.assertNotEqual(cm.exception.retInfo['errCode'], 0, 'Starting the unregistered project by its name should have failed')
    
    self.assertFalse(PVSSBootstrapper.isProjectRegistered(projectName), 'Project appears to be registered')

  def test_dontWait(self):
    self.setGediToOnce(self.projectPath)
    
    PVSSBootstrapper.startProject(projectName, dontWait=True)
    gedi = PVSSManagers.getGedi(projectName)
    self.assertTrue(gedi == None, 'The gedi of the project is already running')
    
    self.setGediToManual(self.projectPath)
    

  # test user/password
  
