#uses "fwBenchmark/fwBenchmark_Util.ctl"
#uses "fwBenchmark/fwBenchmark_Const.ctl"

/*
 * 
 * Functions related to the benchmark of dpCreate()
 * 
 * 
 * jean-charles.tournier@cern.ch
 * CERN - EN/ICE/SCD
 * July 8th, 2015
 * 
 */
 
 /** 
 *
 * Main function launching the benchmark of dpCreate and generating the report
 *  
 * @param iNumDp	(int)	IN	 number of dp to create
 * @param sFile 	(string) IN  file to which the report will be saved
 * @param dfTotalTime (dyn_float) OUT time it took to complete the benchmarck for each round
 * @return value of type 'int' return FWBM_PVSS_SUCCESS in case of success, FWBM_PVSS_SUCCESS in case of error
 */
 int fwBenchmark_dpCreate_Launch( int iNumDp, string sFileName, dyn_float &dfTotalTime)
 {
 	fwBenchmark_Util_clearLog();
 	
 	fwBenchmark_Util_startResourcesMonitoring();
 	
 	for( int i=1; i<=FWBM_NUM_ROUND; i++)
 	{
 		fwBenchmark_dpCreate_Clean();
 		
 		fwBM_log( FWBM_INFO_LOG, __FUNCTION__, "Iteration "+i);
 		float fTotalTime;
 		int iErr = fwBenchmark_dpCreate_Run( iNumDp, fTotalTime);
 		dfTotalTime[i] = fTotalTime;
 		
 		if ( iErr == FWBM_PVSS_ERROR)
 		{
 			fwBenchmark_Util_stopResourcesMonitoring();
 			return FWBM_PVSS_ERROR;
 		}	
 		
 		delay((unsigned)FWBM_DELAY_ROUND);
 	}
 	
 	if(fwBenchmark_Util_stopResourcesMonitoring() == FWBM_PVSS_ERROR)
 	{
 		fwBM_log( FWBM_ERROR_LOG, __FUNCTION__, "Error recovering the reading of the script processes");
 		return FWBM_PVSS_ERROR;
 	}
 	
 	
 	
 	if ( sFileName != "")
 	{
 		if ( fwBenchmark_dpCreate_SaveReport( iNumDp, dfTotalTime, sFileName) != FWBM_PVSS_SUCCESS)
 		{
 			return FWBM_PVSS_ERROR;
 		}	
 	}
 	
 	
 	return FWBM_PVSS_SUCCESS;
 	
 }
 
 /** 
 * Create the DP per say
 * 
 * @param iNumDp	(int)	IN 
 * @param sTotalTime	(float)	IN/OUT 
 * @param sDpBaseName	(string)	IN  is optional with default value '"fw-"'
 * 										We could want create new DPEs with the 'fw-' base
 * 										for the DPE which we want insert a device configuration
 */
 int fwBenchmark_dpCreate_Run( int iNumDp, float &sTotalTime, string sDpBaseName = FWBM_DPBASENAME)
 {
 	time tStart, tEnd;
 	tStart = getCurrentTime();
 	
 	fwBM_log( FWBM_INFO_LOG, __FUNCTION__, "Starting the creation of "+iNumDp+" of type "+FWBM_DPTYPE);
 	
 	if (dpExists(FWBM_DPBASENAME + "1-1-1-1"))
 	{
 		fwBM_log( FWBM_WARNING_LOG, __FUNCTION__, "Some of the DPs already exist");
 		fwBM_log( FWBM_WARNING_LOG, __FUNCTION__, "The benchmark result is may not be relevant.");
 	}
 	
 	for( int i=1; i<=iNumDp; i++)
 	{
 		string sDp = sDpBaseName+i+"-"+i+"-"+i+"-"+i;
 		int iErr = dpCreate(sDp, FWBM_DPTYPE);
 		dyn_errClass deErr = getLastError();
 		if ( iErr != 0 || dynlen(deErr) > 0)
 		{
 			fwBM_log( FWBM_ERROR_LOG, __FUNCTION__, "Error while creating", sDp);
 			fwBM_log( FWBM_ERROR_LOG, __FUNCTION__, deErr);
 			return FWBM_PVSS_ERROR;	
 		}	
 	}
 	
 	tEnd = getCurrentTime();
 	
 	fwBM_log( FWBM_INFO_LOG, __FUNCTION__, "Creation of "+iNumDp+" completed.");
 	
 	sTotalTime = fwBenchmark_Util_computeElapsedTime( tStart, tEnd);
 	
 	fwBM_log( FWBM_INFO_LOG, __FUNCTION__, "Time to complete the dpCreate benchmark: "+sTotalTime+"sec");
 	
 	return FWBM_PVSS_SUCCESS;
 }
 
 void fwBenchmark_dpCreate_Clean(string sDpBaseName = FWBM_DPBASENAME)
 {
 	dyn_string dsDp = dpNames(sDpBaseName + "*");
 	int iNumDp = dynlen(dsDp);
 	
 	fwBM_log( FWBM_INFO_LOG, __FUNCTION__, "Deleting  "+iNumDp+" DPs.");
 	
 	for( int i=1; i<=iNumDp; i++)
 	{
 		
 		dpDelete(dsDp[i]);
 		while( dpExists(dsDp[i]))
 		{
 			delay(0, 100);
 		}
 	}
 	
 	fwBM_log( FWBM_INFO_LOG, __FUNCTION__, "DP deletion completed. Benchmark is ready.");
 }
 
 
 /** 
 * Build and save the report to be generated
 * @param iNumDp	(int)	IN	
 * @param dyn_float	(string)	IN	
 * @param sFileName	(string)	IN	
 * @return value of type 'int'
 */
 int fwBenchmark_dpCreate_SaveReport( int iNumDp, dyn_float dfTotalTime, string sFileName)
 {
 	dyn_string dsCommon = fwBenchmark_Util_getReportCommon( "dpCreate");
 	dyn_string dsContent;
 	
 	dynAppend(dsContent, dsCommon);
 	dynAppend(dsContent, FWBM_NEW_SECTION);
 	dynAppend(dsContent, "Time elapsed to create "+iNumDp+" of type "+FWBM_DPTYPE+":");
 	const int dfTotalTimeLength = dynlen(dfTotalTime);
 	for (int index = 1; index <= dfTotalTimeLength; index++) 
 	{
 	     dynAppend(dsContent, "Round "+index+": "+dfTotalTime[index]);
 	}
 	
 	float fTimeMin = dynMin(dfTotalTime);
 	float fTimeMax = dynMax(dfTotalTime);
 	float fTimeAvg = dynAvg(dfTotalTime);
 	dynAppend(dsContent, "Min: " + fTimeMin);
 	dynAppend(dsContent, "Max: " + fTimeMax);
 	dynAppend(dsContent, "Avg: " + fTimeAvg);
 	
 	fwBM_log( FWBM_INFO_LOG, __FUNCTION__, "Statistics:");
  	fwBM_log( FWBM_INFO_LOG, __FUNCTION__, "Min: "+fTimeMin);
  	fwBM_log( FWBM_INFO_LOG, __FUNCTION__, "Max: "+fTimeMax);
  	fwBM_log( FWBM_INFO_LOG, __FUNCTION__, "Avg: "+fTimeAvg);
  	fwBM_log( FWBM_INFO_LOG, __FUNCTION__, "Report generation completed. Saving report to file.");
 	
 	
 	return fwBenchmark_Util_writeFile( sFileName, dsContent);
 }