#uses "fwBenchmark/fwBenchmark_Util.ctl"
#uses "fwBenchmark/fwBenchmark_Const.ctl"

/*
 * 
 * Functions related to the benchmark of dpCreateLinear()
 * 
 * 
 * jean-charles.tournier@cern.ch
 * CERN - EN/ICE/SCD
 * July 8th, 2015
 * 
 */
 
 /** 
 *
 * Main function launching the benchmark of dpCreate and generating the report
 *  
 * @param iNumDp	(int)	IN	 number of dp to create
 * @param sFile 	(string) IN  file to which the report will be saved
 * @param dfTotalTime (dyn_float) OUT time it took to complete the benchmark. The indexes are per pair, 
 * 										with the first one the number of DP created for a batch and the second one the time needed for this batch
 * @return value of type 'int' return FWBM_PVSS_SUCCESS in case of succes, FWBM_PVSS_SUCCESS in case of error
 */
 int fwBenchmark_dpCreateLinear_Launch( int iNumDp, string sFileName, dyn_dyn_anytype &ddaResult)
 {
 	fwBenchmark_Util_clearLog();
 	dynClear(ddaResult);

 	fwBenchmark_Util_startResourcesMonitoring();
 	int iErr =  fwBenchmark_dpCreateLinear_Run( iNumDp, ddaResult);
	fwBenchmark_Util_stopResourcesMonitoring();

 	if ( iErr == FWBM_PVSS_ERROR)
 	{
 		return FWBM_PVSS_ERROR;
 	}
 	
 	if ( sFileName != "")
 	{
 		if ( fwBenchmark_dpCreateLinear_SaveReport( iNumDp, ddaResult, sFileName) != FWBM_PVSS_SUCCESS)
 		{
 			return FWBM_PVSS_ERROR;
 		}	
 	}
 	
 	
 	return FWBM_PVSS_SUCCESS;
 	
 }


/** 
 * Perform in a loop the batch creation of DPs until 
 * the batch size is equals to iNumDp
 * @param iNumDp	(int)	IN	
 * @param dfTotalTime	(dyn_float)	IN	
 * @return value of type 'int'
 */
int fwBenchmark_dpCreateLinear_Run(int iNumDp, dyn_dyn_anytype &ddaResult) 
{
	
	int iBatchNum = 1;
	
	while( pow(2,iBatchNum) <= (float)iNumDp)
	{
		float fTotalTime;
		float fBunchSize = pow(2,iBatchNum);
		if (fwBenchmark_dpCreateLinear_Core( (int)fBunchSize, (string)iBatchNum, fTotalTime) == FWBM_PVSS_ERROR)
		{
			fwBM_log( FWBM_ERROR_LOG, __FUNCTION__, "Stopping the benchmark due to error");
			return FWBM_PVSS_ERROR;
		}
		
		dyn_anytype daR = makeDynAnytype(fBunchSize, fTotalTime);
		dynAppend(ddaResult, daR);
		iBatchNum++;
	}
	
	return FWBM_PVSS_SUCCESS;
}
 
 
 void fwBenchmark_dpCreateLinear_Clean( )
 {
 	dyn_string dsDp = dpNames("fwLinear-*");
 	int iNumDp = dynlen(dsDp);
 	
 	fwBM_log( FWBM_INFO_LOG, __FUNCTION__, "Starting to delete  "+iNumDp+" DPs.");
 	
 	for( int i=1; i<=iNumDp; i++)
 	{
 		if ( i%100 == 0 )
 		{
 			fwBM_log( FWBM_INFO_LOG, __FUNCTION__, "Number of dp already deleted: "+i);
 		}
 		dpDelete(dsDp[i]);
 		while( dpExists(dsDp[i]))
 		{
 			delay(0, 100);
 		}
 	}
 	
 	fwBM_log( FWBM_INFO_LOG, __FUNCTION__, "DP deletion completed. Benchmark is ready.");
 }
 
 
 /** 
 *
 * Create the DP per say
 *  
 * @param iNumDp	(int)	IN	
 * @param sTotalTime	(float)	IN/OUT	
 * @return value of type 'int'
 */
 int fwBenchmark_dpCreateLinear_Core( int iNumDp, string sBatchId, float &fTotalTime)
 {
 	time tStart, tEnd;
 	tStart = getCurrentTime();
 	
 	fwBM_log( FWBM_INFO_LOG, __FUNCTION__, "Starting the creation of a bunch of  "+iNumDp+" DPs of type "+FWBM_DPTYPE);
	
 	if (dpExists("fwLinear-b"+sBatchId+"-1-1-1"))
 	{
 		fwBM_log( FWBM_WARNING_LOG, __FUNCTION__, "Some of the DPs already exist");
 		fwBM_log( FWBM_WARNING_LOG, __FUNCTION__, "The benchmark result is may not be relevant.");
 	}
 	
 	for( int i=1; i<=iNumDp; i++)
 	{
 		string sDp = "fwLinear-b"+sBatchId+"-"+i+"-"+i+"-"+i+"-"+i;
 	
		int iErr = dpCreate(sDp, FWBM_DPTYPE);
		dyn_errClass deErr = getLastError();
 		if ( iErr != 0 || dynlen(deErr) > 0)
 		{
 			fwBM_log( FWBM_ERROR_LOG, __FUNCTION__, "Error while creating DPs: "+ sDp);
 			fwBM_log( FWBM_ERROR_LOG, __FUNCTION__, deErr);
 			return FWBM_PVSS_ERROR;	
 		}
 	
 	}	
 	
 	tEnd = getCurrentTime();
 	
 	fTotalTime = fwBenchmark_Util_computeElapsedTime( tStart, tEnd);
 	
 	fwBM_log( FWBM_INFO_LOG, __FUNCTION__, "Creation of "+iNumDp+" completed in "+(string)fTotalTime);
 	 	
 	return FWBM_PVSS_SUCCESS;
 }
 
 /** 
 * Build and save the report to be generated
 * @param iNumDp	(int)	IN	
 * @param sTotalTime	(string)	IN	
 * @param ddaResult	(dyn_dyn_anytype)	IN	: array containing for each row, [1] bunch size, [2] time elapsed for the bunch
 * @return value of type 'int'
 */
 int fwBenchmark_dpCreateLinear_SaveReport( int iNumDp, dyn_dyn_anytype ddaResult, string sFileName)
 {
 	dyn_string dsCommon = fwBenchmark_Util_getReportCommon( "dpCreateLinear (max bunch size:"+iNumDp+")");
 	dyn_string dsContent;
 	
 	dynAppend(dsContent, dsCommon);
 	dynAppend(dsContent, FWBM_NEW_SECTION);
 	dynAppend(dsContent, "Bunch size\tElapsed time");
 	int iNumResult = dynlen(ddaResult);
 	for(int i=1; i<=iNumResult; i++)
 	{
 		if (dynlen(ddaResult[i]) >= 2)
 		{
 			dynAppend(dsContent, ddaResult[i][1]+"\t"+ddaResult[i][2]);	
 		} else
 		{
 			fwBM_log( FWBM_WARNING_LOG, __FUNCTION__, "Result seems malformed, the report may be incomplete.");
 		}
 		
 	}
 	
 	return fwBenchmark_Util_writeFile( sFileName, dsContent);
 }