#uses "fwBenchmark/fwBenchmark_Util.ctl"
#uses "fwBenchmark/fwBenchmark_UtilSetGet.ctl"
#uses "fwBenchmark/fwBenchmark_Const.ctl"

/*
 * 
 * Functions related to the benchmark of dpGetAttributes
 * 
 * jean-charles.tournier@cern.ch
 * CERN - EN/ICE/SCD
 * July 8th, 2015
 * 
 * 
 * alejandro.herrero.de.campos@cern.ch
 * CERN - EN/ICE/SCD
 * November 3th, 2015
 * 
 */
 
 
 /** 
 * Main function launching the benchmark of dpGetAttributes and generating the report
 * @param iNumAttributes	(int)	IN number of attributes to get
 * @param iBunchStep	(int)	IN 	difference in size between two bunches
 * @param sConfiguration	(string)	IN  configurations names to set and get values (fwBenchmark_Const.ctl)
 * @param sFileName	(string)	IN file to which the report will be saved
 * @param ddaResult	(dyn_dyn_anytype)	IN/OUT list of result with [1] bunch size and [2] time taken 
 * 													to perform dpGet on the iNumDp by bunches of iBunchStep
 * @return value of type 'int' FWBM_PVSS_SUCCESS in case of succes, FWBM_PVSS_SUCCESS in case of error
 */
 int fwBenchmark_dpGetAttributes_Launch( int iNumAttributes, int iBunchStep, string sConfiguration, string sFileName, dyn_dyn_anytype &ddaResult)
 {
 	fwBenchmark_Util_clearLog();
 	dynClear(ddaResult);
 	
 	//Set the DPE number to be considerer depending of the configuration
 	//and the bunchStep will also change
 	int iNumDp = 0;
 	if(fwBenchmark_UtilSetGet_DpsToHandle(sConfiguration, iNumAttributes, iNumDp, iBunchStep) == FWBM_PVSS_ERROR)
 		return FWBM_PVSS_ERROR;
 	
 	//Do the iterations and run
 	dyn_dyn_float ddaResultIter;
 	
 	dyn_float dfBunchSize;
 	dyn_dyn_float ddfTotalTimePerBunch;
 	dyn_float dfExecutionTime;
 	
 	float fElapsedTime;
 	
 	fwBenchmark_Util_startResourcesMonitoring(); 
 	for( int i=1; i<=FWBM_NUM_ROUND; i++)
 	{
 		fwBM_log( FWBM_INFO_LOG, __FUNCTION__, "Iteration "+i);
 		
 		dynClear(ddaResultIter); 		
 		fElapsedTime = 0;
 		int iErr = fwBenchmark_dpGetAttributes_Run( iNumAttributes, iNumDp, iBunchStep, sConfiguration, 		//in params
 													ddaResultIter,fElapsedTime); 							//out params
 		
		fwBenchmark_UtilSetGet_BuildVectorsToWrite(i, ddaResultIter, fElapsedTime, 					//in params
 													dfBunchSize, ddfTotalTimePerBunch,dfExecutionTime);		//out params
 		
 		if ( iErr == FWBM_PVSS_ERROR)
 		{
 			fwBenchmark_Util_stopResourcesMonitoring();
 			return FWBM_PVSS_ERROR;
 		}	
 		delay((unsigned)FWBM_DELAY_ROUND);
 	}
 	if(fwBenchmark_Util_stopResourcesMonitoring() == FWBM_PVSS_ERROR)
 	{
 		fwBM_log( FWBM_ERROR_LOG, __FUNCTION__, "Error recovering the reading of the script processes");
 		return FWBM_PVSS_ERROR;
 	}
 	
 	//Return the result of the last iteration
	//This result is for the UI windows so, it's not very import the average
	ddaResult = ddaResultIter;	 
 	
 	if ( sFileName != "")
 	{
 		string reportName = "dpGet " + sConfiguration;
 		if ( fwBenchmark_UtilSetGet_SaveReport( iNumAttributes, iNumDp, iBunchStep, sConfiguration, reportName, dfBunchSize, ddfTotalTimePerBunch, dfExecutionTime, sFileName) != FWBM_PVSS_SUCCESS)
 		{
 			return FWBM_PVSS_ERROR;
 		}	
 	}
 	
 	
 	return FWBM_PVSS_SUCCESS;
 	
 }
 
 /** 
 * Prepare the DP to be set and asked the DP to be retrieved
 * @param iNumAttributes	(int)	IN number of attributes to be retrieved
 * @param iNumDp	(int)	IN 	number of DPEs created to do the retrieved (if the configuration has
 * 								only one attribute, this number will be the same of number of attributes)
 * @param iBunchStep	(int)	IN increment for each new bunch
 * @param sConfiguration	(string)	IN Configurations names to set and get values (fwBenchmark_Const.ctl)
 * @param ddaResult	(dyn_dyn_anytype)	IN/OUT ist of result with [1] bunch size and [2] time taken 
 * 													to perform dpGet on the iNumDp by bunches of iBunchStep
 * @param fElapsedTime	(float)	IN/OUT elapsed time to perform the test
 * @return value of type 'int' in case of succes, FWBM_PVSS_SUCCESS in case of error
 */
int fwBenchmark_dpGetAttributes_Run(int iNumAttributes, int iNumDp, int iBunchStep, string sConfiguration, dyn_dyn_anytype &ddaResult, float &fElapsedTime) 
{
	time tStart, tEnd;
 	tStart = getCurrentTime();
 	
	string sDpBaseName = FWBM_DPBASENAME + sConfiguration + "-";
	dyn_string dsDpeValueExisting = dpNames(sDpBaseName + "*.");
	dyn_string dsDpeValue;
	int iNumDpExisting = dynlen(dsDpeValueExisting);
	
	if (iNumDpExisting<iNumDp)
	{
		fwBM_log( FWBM_ERROR_LOG, __FUNCTION__, "The number of existing DP is smaller than the number of DP asked to be retrieved - Stopping.");
		fwBM_log( FWBM_ERROR_LOG, __FUNCTION__, "\tNumber of requested DPs: "+iNumDp);
		fwBM_log( FWBM_ERROR_LOG, __FUNCTION__, "\tNumber of existing DPs: "+iNumDpExisting);
 		return FWBM_PVSS_ERROR;
	}
	
	for( int i=1; i<=iNumDp; i++)
	{
		dsDpeValue[i] = dsDpeValueExisting[i];
	}
	
	//Some initial conditions
	int iBunchSize = 0;
	int err = fwBenchmark_UtilSetGet_InitialConditionsOfAttributes(sConfiguration,iNumAttributes, iBunchStep, iBunchSize);
	if(err == FWBM_PVSS_ERROR) return FWBM_PVSS_ERROR;
	
	//Bunches loop, which increase the bunch size each iteration (according a bunch step)
	while( iBunchSize < iNumAttributes)
	{
        fwBM_log( FWBM_INFO_LOG, __FUNCTION__, "Retrieving "+iNumAttributes+" attributes in "+iNumDp+" DPs in the '" + 
                sConfiguration+ "' configuration by bunches of "+iBunchSize);
		dyn_anytype daResult;
		if ( fwBenchmark_dpGetAttributes_Core( dsDpeValue, iNumAttributes, iBunchSize, sConfiguration, daResult) == FWBM_PVSS_ERROR)
 		{
 			fwBM_log( FWBM_ERROR_LOG, __FUNCTION__, "Stopping the benchmark due to error");
 			return FWBM_PVSS_ERROR;
 		}
 		iBunchSize = iBunchSize + iBunchStep;
 		dynAppend( ddaResult, daResult);
	}
	
	//now check if the dpGet on all DPs at once has been done
	if (iBunchSize>=iNumAttributes)
	{
		iBunchSize = iNumAttributes;
        fwBM_log( FWBM_INFO_LOG, __FUNCTION__, "Retrieving "+iNumAttributes+" attributes (All attributes at once) in "+iNumDp+" DPs in the '" + 
                sConfiguration+ "' configuration by bunches of "+iBunchSize);
		dyn_anytype daResult;
		if ( fwBenchmark_dpGetAttributes_Core( dsDpeValue, iNumAttributes, iBunchSize, sConfiguration, daResult) == FWBM_PVSS_ERROR)
 		{
 			fwBM_log( FWBM_ERROR_LOG, __FUNCTION__, "Stopping the benchmark due to error");
 			return FWBM_PVSS_ERROR;
 		}
 		dynAppend( ddaResult, daResult);
	}
	
	//get the total time to complete the test and add to the vector result
	tEnd = getCurrentTime();
	fElapsedTime =  fwBenchmark_Util_computeElapsedTime( tStart, tEnd);
	
	return FWBM_PVSS_SUCCESS;	
}
 
/** 
 * Retrieve the value of dsDpeValue by bunches of iBunchSize and store
 * the result in daResult with [1] the bunch size and [2] the time it took to do the dpGet
 * @param dsDpeValue	(dyn_string)	IN 
 * @param iBunchSize	(int)	IN 
 * @param sConfiguration	(string)	IN 
 * @param daResult	(dyn_anytype)	IN/OUT 
 * @return value of type 'int'
 */
int fwBenchmark_dpGetAttributes_Core(dyn_string dsDpeValue, int iNumAttributes, int iBunchSize,string sConfiguration, dyn_anytype &daResult) 
{
	int iNumIteration = iNumAttributes/iBunchSize;

	dyn_string dsConfigurationNames;
	dyn_anytype daConfigurationValues;
	
	//Link the attribute name with the value, depending of the configuration
	fwBenchmark_UtilSetGet_GetNameValueConfiguration(sConfiguration, 	dsConfigurationNames, daConfigurationValues);
	
	float fTotalTime = 0;
	
	int iIndexDpe = 1;
	int iIndexConfig = 1;
	
	for( int i=1; i<=iNumIteration; i++)
	{
		//build the bunch
		dyn_string dsDpBunch;
		dynClear(dsDpBunch);
		
		for( int j = 1; j <= iBunchSize; j++)
		{		
			int iIndexLoop = j;
			//Build the DPs bunch
			dsDpBunch[iIndexLoop] = dsDpeValue[iIndexDpe] + dsConfigurationNames[iIndexConfig];
			
			//Index update
			_fwBenchmark_dpGetAttributes_BuildingBunchIndexUpdate(sConfiguration, iIndexLoop, iIndexDpe, iIndexConfig);
		}
		
		//perform the actual dpGet
		float fElaspedTime = 0;
		if (_fwBenchmark_dpGetAttributes_CoreHelper(dsDpBunch, sConfiguration, fElaspedTime) != FWBM_PVSS_SUCCESS)
		{
			return FWBM_PVSS_ERROR;
		}
		fTotalTime = fTotalTime + fElaspedTime;	
		
	}
	
	//and now do for the rest of the DP which can't make a full bunch
	if(iNumAttributes>=iNumIteration*iBunchSize + 1)
	{	
		//build the bunch
		dyn_string dsDpBunch;
		dynClear(dsDpBunch);
		
		int iIndexStart = iNumIteration*iBunchSize + 1;
		for( int j = iIndexStart; j <= iNumAttributes; j++)
		{
			int iIndexLoop = (j-iIndexStart)+1;

			//Build the DPs bunch
			dsDpBunch[iIndexLoop] = dsDpeValue[iIndexDpe] + dsConfigurationNames[iIndexConfig];
			
			//Index update
			_fwBenchmark_dpGetAttributes_BuildingBunchIndexUpdate(sConfiguration, iIndexLoop, iIndexDpe, iIndexConfig);

		}
		
		//perform the actual dpGet
		if ( dynlen(dsDpBunch))
		{
			float fElaspedTime = 0;
			if (_fwBenchmark_dpGetAttributes_CoreHelper(dsDpBunch, sConfiguration, fElaspedTime) != FWBM_PVSS_SUCCESS)
			{
				return FWBM_PVSS_ERROR;
			}
			fTotalTime = fTotalTime + fElaspedTime;
		}
	}
	
	//and finally store the result
	daResult[1] = iBunchSize; 
	daResult[2] = fTotalTime;
	
	return FWBM_PVSS_SUCCESS;
}

/** 
 * Update the index of DPEs and the Configuration to access correctly in the bunch building
 * @param sConfiguration	(string)	IN 
 * @param iIndexLoop	(int)	IN 
 * @param iIndexDpe	(int)	IN/OUT 
 * @param iIndexConfig	(int)	IN/OUT 
 * @return value of type 'int'
 */
int _fwBenchmark_dpGetAttributes_BuildingBunchIndexUpdate(string sConfiguration, int iIndexLoop, int &iIndexDpe, int &iIndexConfig)
{
	switch(sConfiguration)
	{
		case FWBM_CONFIG_ORIGINALVALUE:
			//Original value
			iIndexDpe++;
			iIndexConfig = 1;
			break;
		
		case FWBM_CONFIG_ARCHIVE:
			//Archive
			if(iIndexConfig == dynlen(FWBM_ARCHIVE_CONFIG_NAME))
			{
				iIndexDpe++;
				iIndexConfig = 1;
			}
			else
			{
				iIndexConfig++;
			}
			break;
			
		case FWBM_CONFIG_ALERTHDL:
			//Alert Handling
			if(iIndexConfig == dynlen(FWBM_ALERTHDL_CONFIG_NAME))
			{
				iIndexDpe++;
				iIndexConfig = 1;
			}
			else
			{
				iIndexConfig++;
			}
			break;
			
		case FWBM_CONFIG_DRIVER_SIM:
		case FWBM_CONFIG_DRIVER_S7:
		case FWBM_CONFIG_DRIVER_MODBUS:
		case FWBM_CONFIG_DRIVER_IEC:
			//Devices
			if(iIndexConfig == dynlen(FWBM_DEVICE_CONFIG_NAME))
			{
				iIndexDpe++;
				iIndexConfig = 1;
			}
			else
			{
				iIndexConfig++;
			}
			break;
		
		default:
			return FWBM_PVSS_ERROR;
				
	}
	
	return FWBM_PVSS_SUCCESS;	
}


/** 
 * Helper function to dpGetAttributes_Core to actually do the dpGet and measure the time it takes
 * @param dsDp	(dyn_string)	IN 
 * @param sConfiguration	(string)	IN 
 * @param fElapsedTime	(float)	IN/OUT time it took to complete the benchmark. The indexes are per pair, 
 * 										with the first one the number of DP created for a batch and
 * 										the second one the time needed for this batch
 * @return value of type 'int'
 */
private int _fwBenchmark_dpGetAttributes_CoreHelper( dyn_string dsDp, string sConfiguration, float &fElapsedTime)
{
	//Rename the DPNames with the parameters to obtain and put a value
	//In _original.._value there is only a attribute per DPE.
	//In other case, the new dsDp will be as longer as the old dsDp * iNumberAttributes
		
	dyn_anytype daValue;
	time tStart, tEnd;
 	tStart = getCurrentTime();
		
	//perform the actual dpGet
	int iErr = dpGet(dsDp, daValue);
	dyn_errClass deErr = getLastError();
	
	if ( iErr != 0 || dynlen(deErr)>0)
	{
		fwBM_log( FWBM_ERROR_LOG, __FUNCTION__, "Error while performing a dpGet on "+dynlen(dsDp)+" DPEs.");
		fwBM_log( FWBM_ERROR_LOG, __FUNCTION__, deErr);
		return FWBM_PVSS_ERROR;
	}
	
		
	tEnd = getCurrentTime();
	fElapsedTime =  fwBenchmark_Util_computeElapsedTime( tStart, tEnd);
	
	return FWBM_PVSS_SUCCESS;
}
