#uses "fwBenchmark/fwBenchmark_Util.ctl"
#uses "fwBenchmark/fwBenchmark_Const.ctl"

/*
 * 
 * Functions related to the benchmark of dpGetValue
 * 
 * 
 * jean-charles.tournier@cern.ch
 * CERN - EN/ICE/SCD
 * July 8th, 2015
 * 
 */
 
 /** 
 *
 * Main function launching the benchmark of dpCreate and generating the report
 *  
 * @param iNumDp	(int)	IN	 number of dp to create
 * @param sFile 	(string) IN  file to which the report will be saved
 * @param iBunchStep	(int)	IN	 difference in size between two bunches
 * @param dfTotalTime (dyn_float) OUT time it took to complete the benchmark. The indexes are per pair, 
 * 										with the first one the number of DP created for a batch and the second one the time needed for this batch
 * @return value of type 'int' return FWBM_PVSS_SUCCESS in case of succes, FWBM_PVSS_SUCCESS in case of error
 */
 int fwBenchmark_dpGetValue_Launch( int iNumDp, int iBunchStep, string sFileName, dyn_dyn_anytype &ddaResult)
 {
 	fwBenchmark_Util_clearLog();
 	dynClear(ddaResult);
 	
 	fwBenchmark_Util_startResourcesMonitoring(); 	
 	int iErr = fwBenchmark_dpGetValue_Run( iNumDp, iBunchStep, ddaResult);
 	fwBenchmark_Util_stopResourcesMonitoring();
 	 
 	if ( iErr == FWBM_PVSS_ERROR)
 	{
 		return FWBM_PVSS_ERROR;
 	}
 	
 	
 	if ( sFileName != "")
 	{
 		if ( fwBenchmark_dpGetValue_SaveReport( iNumDp, iBunchStep, ddaResult, sFileName) != FWBM_PVSS_SUCCESS)
 		{
 			return FWBM_PVSS_ERROR;
 		}	
 	}
 	
 	
 	return FWBM_PVSS_SUCCESS;
 	
 }

/** 
 * Prepare the DP to be retrieved and asked the DP to be retrieved 
 * @param iNumDp	(int)	IN	: number of DP total to be retrieved
 * @param iBunchStep	(int)	IN	: increment for each new bunch
 * @param ddaResult	(dyn_dyn_anytype)	IN/OUT	: list of result with [1] bunch size and [2] time taken 
 * 													to perform dpGet on the iNumDp by bunches of iBunchStep 
 * @return value of type 'int' FWBM_PVSS_SUCCESS or FWBM_PVSS_ERROR
 */
int fwBenchmark_dpGetValue_Run(int iNumDp, int iBunchStep, dyn_dyn_anytype &ddaResult) 
{
	dyn_string dsDpeValueExisting = dpNames("fw-*.");
	dyn_string dsDpeValue;
	int iNumDpExisting = dynlen(dsDpeValueExisting);
	
	if (iNumDpExisting<iNumDp)
	{
		fwBM_log( FWBM_ERROR_LOG, __FUNCTION__, "The number of existing DP is smaller than the number of DP asked to be retrieved - Stopping.");
		fwBM_log( FWBM_ERROR_LOG, __FUNCTION__, "\tNumber of requested DPs: "+iNumDp);
		fwBM_log( FWBM_ERROR_LOG, __FUNCTION__, "\tNumber of existing DPs: "+iNumDpExisting);
 		return FWBM_PVSS_ERROR;
	}
	
	for( int i=1; i<=iNumDp; i++)
	{
		dsDpeValue[i] = dsDpeValueExisting[i];
	}
	
	int iBunchSize = 1;
	while( iBunchSize <= iNumDp)
	{
		fwBM_log( FWBM_INFO_LOG, __FUNCTION__, "Retrieving "+iNumDp+" DPs by bunches of "+iBunchSize);
		dyn_anytype daResult;
		if ( fwBenchmark_dpGetValue_Core( dsDpeValue, iBunchSize, daResult) == FWBM_PVSS_ERROR)
 		{
 			fwBM_log( FWBM_ERROR_LOG, __FUNCTION__, "Stopping the benchmark due to error");
 			return FWBM_PVSS_ERROR;
 		}
 		iBunchSize = iBunchSize + iBunchStep;
 		dynAppend( ddaResult, daResult);
	}
	
	//now check if the dpGet on all DPs at once has been done
	if (iBunchSize>iNumDp)
	{
		iBunchSize = iNumDp;
		fwBM_log( FWBM_INFO_LOG, __FUNCTION__, "Retrieving "+iNumDp+" DPs by bunches of "+iBunchSize);
		dyn_anytype daResult;
		if ( fwBenchmark_dpGetValue_Core( dsDpeValue, iBunchSize, daResult) == FWBM_PVSS_ERROR)
 		{
 			fwBM_log( FWBM_ERROR_LOG, __FUNCTION__, "Stopping the benchmark due to error");
 			return FWBM_PVSS_ERROR;
 		}
 		iBunchSize = iBunchSize + iBunchStep;
 		dynAppend( ddaResult, daResult);
	}
	
	return FWBM_PVSS_SUCCESS;	
}

/** 
 * Retrieve the value of dsDpeValue by bunches of iBunchSize and store
 * the result in daResult with [1] the bunch size and [2] the time it took to do the dpGet
 * @param dsDpeValue	(dyn_string)	IN	
 * @param iBunchSize	(int)	IN	
 * @param daResult	(dyn_anytype)	IN	
 * @return value of type 'int'
 */
int fwBenchmark_dpGetValue_Core(dyn_string dsDpeValue, int iBunchSize, dyn_anytype &daResult) 
{
	int iNumDp = dynlen(dsDpeValue);
	int iNumIteration = iNumDp/iBunchSize;
	
	float fTotalTime = 0;
	
	for( int i=1; i<=iNumIteration; i++)
	{
		//build the bunch
		dyn_string dsDpBunch;
		for( int j = 1; j <= iBunchSize; j++)
		{
			dsDpBunch[j] = dsDpeValue[(i-1)*iBunchSize+j];
		}
		
		//perform the actual dpGet
		float fElaspedTime = 0;
		if (_fwBenchmark_dpGetValue_CoreHelper(dsDpBunch, fElaspedTime) != FWBM_PVSS_SUCCESS)
		{
			return FWBM_PVSS_ERROR;
		}
		fTotalTime = fTotalTime + fElaspedTime;	
		
	}
	
	//and now do for the rest of the DP which can't make a full bunch
	dyn_string dsDpBunch;
	int j=1;
	for( int i=iNumIteration*iBunchSize; i<=iNumDp; i++)
	{
		dsDpBunch[j] = dsDpeValue[i];
		j++;	
	}
	
	if ( dynlen(dsDpBunch))
	{
		float fElaspedTime = 0;
		if (_fwBenchmark_dpGetValue_CoreHelper(dsDpBunch, fElaspedTime) != FWBM_PVSS_SUCCESS)
		{
			return FWBM_PVSS_ERROR;
		}
		fTotalTime = fTotalTime + fElaspedTime;
	}
	
	//and finally store the result
	daResult[1] = iBunchSize; 
	daResult[2] = fTotalTime;
	
	return FWBM_PVSS_SUCCESS;
}

/** 
 * Helper function to dpGetValue_Core to actually do the dpGet and measure the time it takes
 * @param dsDp	(dyn_string)	IN	
 * @param fElapsedTime	(float)	IN/OUT	
 * @return value of type 'int'
 */
private int _fwBenchmark_dpGetValue_CoreHelper( dyn_string dsDp, float &fElapsedTime)
{
	dyn_anytype daValue;
	time tStart, tEnd;
 	tStart = getCurrentTime();
		
	//perform the actual dpGet
	int iErr = dpGet(dsDp, daValue);
	dyn_errClass deErr = getLastError();
	
	if ( iErr != 0 || dynlen(deErr)>0)
	{
		fwBM_log( FWBM_ERROR_LOG, __FUNCTION__, "Error while performing a dpGet on "+dynlen(dsDp)+" DPEs.");
		fwBM_log( FWBM_ERROR_LOG, __FUNCTION__, deErr);
		return FWBM_PVSS_ERROR;
	}
	
	
		
	tEnd = getCurrentTime();
	fElapsedTime =  fwBenchmark_Util_computeElapsedTime( tStart, tEnd);
	
	return FWBM_PVSS_SUCCESS;
}

/** 
 * Generate the report at the end of the benchmark
 * 
 * @param iNumDp	(int)	IN: number of DP on which the function has been performed 
 * @param iBunchStep	(int)	IN: difference in size between two bunches
 * @param ddaResult	(dyn_dyn_anytype)	IN: two dimensions array with the first column being the bunch size 
 * 											and the second the time taken to perform the function on iNumDp
 * @param sFileName	(string)	IN: name of the file in which the report will be stored
 * @return value of type 'int': FWBM_PVSS_SUCCESS in case of success, FWBM_PVSS_ERROR otherwise
 */
int fwBenchmark_dpGetValue_SaveReport(int iNumDp, int iBunchStep, dyn_dyn_anytype ddaResult, string sFileName) 
{
  fwBM_log( FWBM_INFO_LOG, __FUNCTION__, "Generating the report...");
	 dyn_string dsCommon = fwBenchmark_Util_getReportCommon( "dpGetValue (Total number of DPs: "+iNumDp+", Bunch increment: "+iBunchStep+"))");
 	dyn_string dsContent;
 	
 	int iMinBunchSize = -1;
 	int iMaxBunchSize = -1;
 	float fMinExecTime = maxFLOAT();
 	float fMaxExecTime = -1;
 	
 	dynAppend(dsContent, dsCommon);
 	dynAppend(dsContent, FWBM_NEW_SECTION);
  
 	dynAppend(dsContent, "Bunch size\tElapsed time");
 	int iNumResult = dynlen(ddaResult);
 	for(int i=1; i<=iNumResult; i++)
 	{
 		if (dynlen(ddaResult[i]) >= 2)
 		{
 			int iBunchSize = ddaResult[i][1];
 			float fExecTime = ddaResult[i][2];
 			dynAppend(dsContent, iBunchSize+"\t"+fExecTime);	

    //get the min and max values
    if ( fExecTime > fMaxExecTime)
    {
      fMaxExecTime = fExecTime;
      iMaxBunchSize = iBunchSize;
    }
    
    if ( fExecTime < fMinExecTime)
    {
      fMinExecTime = fExecTime;
      iMinBunchSize = iBunchSize;
    }
    
 		} else
 		{
 			fwBM_log( FWBM_WARNING_LOG, __FUNCTION__, "Result seems malformed, the report may be incomplete.");
 		}
 		
 	}
 	
 	dynAppend(dsContent, FWBM_NEW_SECTION);
 	dynAppend(dsContent, "Statistics");
  string sStatMax = "Max Elapsed Time: "+fMaxExecTime+" ( "+iMaxBunchSize+" bunches)";  
  string sStatMin = "Min Elapsed Time: "+fMinExecTime+" ( "+iMinBunchSize+" bunches)";
  dynAppend(dsContent, sStatMax);
  dynAppend(dsContent, sStatMin); 	
  
  fwBM_log( FWBM_INFO_LOG, __FUNCTION__, "Statistics:");
  fwBM_log( FWBM_INFO_LOG, __FUNCTION__, sStatMax);
  fwBM_log( FWBM_INFO_LOG, __FUNCTION__, sStatMin);
  fwBM_log( FWBM_INFO_LOG, __FUNCTION__, "Report generation completed. Saving report to file.");
  
 	return fwBenchmark_Util_writeFile( sFileName, dsContent);
}
