#uses "fwBenchmark/fwBenchmark_Util.ctl"
#uses "fwBenchmark/fwBenchmark_UtilSetGet.ctl"
#uses "fwBenchmark/fwBenchmark_Const.ctl"
#uses "fwBenchmark/fwBenchmark_DpCreate.ctl"

/*
 * 
 * Functions related to the benchmark of dpSetAttributes
 * 
 * jean-charles.tournier@cern.ch
 * CERN - EN/ICE/SCD
 * July 8th, 2015
 * 
 * 
 * alejandro.herrero.de.campos@cern.ch
 * CERN - EN/ICE/SCD
 * October 29th, 2015
 * 
 */
 
  const int FWBM_DPSETSCALAR_NUM_BUNCHES_MAX = 5000;
  
 /** 
 * Main function launching the benchmark of dpSetAttributes and generating the report
 * @param iNumAttributes	(int)	IN number of attributes to set
 * @param iBunchStep	(int)	IN 	difference in size between two bunches
 * @param sConfiguration	(string)	IN  configurations names to set and get values (fwBenchmark_Const.ctl)
 * @param sFileName	(string)	IN file to which the report will be saved
 * @param ddaResult	(dyn_dyn_anytype)	IN/OUT list of result with [1] bunch size and [2] time taken 
 * 													to perform dpSet on the iNumDp by bunches of iBunchStep
 * @return value of type 'int' FWBM_PVSS_SUCCESS in case of succes, FWBM_PVSS_SUCCESS in case of error
 */
 int fwBenchmark_dpSetAttributes_Launch( int iNumAttributes, int iBunchStep, string sConfiguration, string sFileName, dyn_dyn_anytype &ddaResult)
 {
 	fwBenchmark_Util_clearLog();
 	dynClear(ddaResult);
 	
 	//Set the DPE number to be considerer depending of the configuration
 	//and the bunchStep will also change
 	int iNumDp = 0;
 	if(fwBenchmark_UtilSetGet_DpsToHandle(sConfiguration, iNumAttributes, iNumDp, iBunchStep) == FWBM_PVSS_ERROR)
 		return FWBM_PVSS_ERROR;
 	
 	//Create new DPEs (necessary for each configuration, at least, the devices configurations)
	float fNotUsed;
	string sDpBaseName = FWBM_DPBASENAME + sConfiguration + "-";
 	fwBenchmark_dpCreate_Run( iNumDp, fNotUsed, sDpBaseName);
 	
 	//Do the iterations and run
 	dyn_dyn_float ddaResultIter;
 	
 	dyn_float dfBunchSize;
 	dyn_dyn_float ddfTotalTimePerBunch;
 	dyn_float dfExecutionTime;
 	
 	float fElapsedTime;
 	
 	fwBenchmark_Util_startResourcesMonitoring(); 
 	for( int i=1; i<=FWBM_NUM_ROUND; i++)
 	{
 		fwBM_log( FWBM_INFO_LOG, __FUNCTION__, "Iteration "+i);
 		
 		dynClear(ddaResultIter); 		
 		fElapsedTime = 0;
 		
 		int iErr = fwBenchmark_dpSetAttributes_Run(iNumAttributes, iNumDp, iBunchStep, sConfiguration, 		//in params
 													ddaResultIter,fElapsedTime);							//out params
 													
		fwBenchmark_UtilSetGet_BuildVectorsToWrite(i, ddaResultIter, fElapsedTime,					//in params
 													dfBunchSize, ddfTotalTimePerBunch,dfExecutionTime);		//out params										
 		if ( iErr == FWBM_PVSS_ERROR)
 		{
 			fwBenchmark_Util_stopResourcesMonitoring();
 			return FWBM_PVSS_ERROR;
 		}	
 		delay((unsigned)FWBM_DELAY_ROUND);
 	}
 	if(fwBenchmark_Util_stopResourcesMonitoring() == FWBM_PVSS_ERROR)
 	{
 		fwBM_log( FWBM_ERROR_LOG, __FUNCTION__, "Error recovering the reading of the script processes");
 		return FWBM_PVSS_ERROR;
 	}
 	
 	//Return the result of the last iteration
	//This result is for the UI windows so, it's not very import the average
	ddaResult = ddaResultIter;	 
 	
 	//Save
 	if ( sFileName != "")
 	{
 		string reportName = "dpSet " + sConfiguration;
 		if ( fwBenchmark_UtilSetGet_SaveReport( iNumAttributes, iNumDp, iBunchStep, sConfiguration, reportName, dfBunchSize, ddfTotalTimePerBunch, dfExecutionTime, sFileName) != FWBM_PVSS_SUCCESS)
 		{
 			return FWBM_PVSS_ERROR;
 		}	
 	}
 	
 	return FWBM_PVSS_SUCCESS;
 	
 }
 
 /** 
 * Prepare the DP to be set and asked the DP to be set
 * @param iNumAttributes	(int)	IN number of attributes to be set
 * @param iNumDp	(int)	IN 	number of DPEs created to do the set (if the configuration has
 * 								only one attribute, this number will be the same of number of attributes)
 * @param iBunchStep	(int)	IN increment for each new bunch
 * @param sConfiguration	(string)	IN Configurations names to set and get values (fwBenchmark_Const.ctl)
 * @param ddaResult	(dyn_dyn_anytype)	IN/OUT ist of result with [1] bunch size and [2] time taken 
 * 													to perform dpSet on the iNumDp by bunches of iBunchStep
 * @param fElapsedTime	(float)	IN/OUT elapsed time to perform the test
 * @return value of type 'int' in case of succes, FWBM_PVSS_SUCCESS in case of error
 */
int fwBenchmark_dpSetAttributes_Run(int iNumAttributes, int iNumDp, int iBunchStep, string sConfiguration, dyn_dyn_anytype &ddaResult, float &fElapsedTime) 
{
	time tStart, tEnd;
 	tStart = getCurrentTime();
 	
	string sDpBaseName = FWBM_DPBASENAME + sConfiguration + "-";
	dyn_string dsDpeValueExisting = dpNames(sDpBaseName + "*.");
	dyn_string dsDpeValue;
	int iNumDpExisting = dynlen(dsDpeValueExisting);
	
	if (iNumDpExisting<iNumDp)
	{
		fwBM_log( FWBM_ERROR_LOG, __FUNCTION__, "The number of existing DP is smaller than the number of DP asked to be retrieved - Stopping.");
		fwBM_log( FWBM_ERROR_LOG, __FUNCTION__, "\tNumber of requested DPs: "+iNumDp);
		fwBM_log( FWBM_ERROR_LOG, __FUNCTION__, "\tNumber of existing DPs: "+iNumDpExisting);
 		return FWBM_PVSS_ERROR;
	}
	
	for( int i=1; i<=iNumDp; i++)
	{
		dsDpeValue[i] = dsDpeValueExisting[i];
	}
	
	//Some initial conditions
	int iBunchSize = 0;
	int err = fwBenchmark_UtilSetGet_InitialConditionsOfAttributes(sConfiguration,iNumAttributes, iBunchStep, iBunchSize);
	if(err == FWBM_PVSS_ERROR) return FWBM_PVSS_ERROR;
	
	//Bunches loop, which increase the bunch size each iteration (according a bunch step)
    while(iBunchSize < iNumAttributes && iBunchSize <= FWBM_DPSETSCALAR_NUM_BUNCHES_MAX)              
    {
        fwBM_log( FWBM_INFO_LOG, __FUNCTION__, "Setting "+iNumAttributes+" attributes in "+iNumDp+" DPs in the '" + 
                sConfiguration+ "' configuration by bunches of "+iBunchSize);
        dyn_anytype daResult;
        if ( fwBenchmark_dpSetAttributes_Core( dsDpeValue, iNumAttributes, iBunchSize, sConfiguration,daResult) == FWBM_PVSS_ERROR)
        {
                fwBM_log( FWBM_ERROR_LOG, __FUNCTION__, "Stopping the benchmark due to error");
                return FWBM_PVSS_ERROR;
        }
        iBunchSize = iBunchSize + iBunchStep;
        dynAppend( ddaResult, daResult);
    }
	
    //now check if the dpSet on all Attributes at once has been done
    if (iBunchSize>=iNumAttributes || iBunchSize>FWBM_DPSETSCALAR_NUM_BUNCHES_MAX)
    {
        iBunchSize = iNumAttributes;
        fwBM_log( FWBM_INFO_LOG, __FUNCTION__, "Setting "+iNumAttributes+" attributes (All attributes at once) in "+iNumDp+" DPs in the '" + 
                sConfiguration+ "' configuration by bunches of "+iBunchSize);
        dyn_anytype daResult;
        if ( fwBenchmark_dpSetAttributes_Core( dsDpeValue, iNumAttributes, iBunchSize, sConfiguration, daResult) == FWBM_PVSS_ERROR)
        {
            fwBM_log( FWBM_ERROR_LOG, __FUNCTION__, "Stopping the benchmark due to error");
            return FWBM_PVSS_ERROR;
        }
        dynAppend( ddaResult, daResult);
    }
	
	//get the total time to complete the test
	tEnd = getCurrentTime();
	fElapsedTime =  fwBenchmark_Util_computeElapsedTime( tStart, tEnd);
	
	return FWBM_PVSS_SUCCESS;	
}

/** 
 * 
 * Set the value of dsDpeValue by bunches of iBunchSize and store
 * the result in daResult with [1] the bunch size and [2] the time it took to do the dpGet
 * 
 * @param dsDpeValue	(dyn_string)	IN 
 * @param iBunchSize	(int)	IN 
 * @param sConfiguration	(string)	IN 
 * @param daResult	(dyn_anytype)	IN/OUT 
 * @return value of type 'int'
 */
int fwBenchmark_dpSetAttributes_Core(dyn_string dsDpeValue, int iNumAttributes, int iBunchSize, string sConfiguration, dyn_anytype &daResult) 
{
	int iNumIteration = iNumAttributes/iBunchSize;

	dyn_string dsConfigurationNames;
	dyn_anytype daConfigurationValues;
	
	//Link the attribute name with the value, depending of the configuration
	fwBenchmark_UtilSetGet_GetNameValueConfiguration(sConfiguration, 	dsConfigurationNames, daConfigurationValues);
	
	float fTotalTime = 0;
	
	int iIndexDpe = 1;
	int iIndexConfig = 1;
	
	dyn_string dsDpBunch;
	dyn_anytype dsAttributesBunch;

	for( int i=1; i<=iNumIteration; i++)
	{
		//Build the bunch
		dynClear(dsDpBunch);
		dynClear(dsAttributesBunch);
		for( int j = 1; j <= iBunchSize; j++)
		{
			int iIndexLoop = j;
			//Build the attributes value bunch
			string sReferenceName = "";
			if(dynlen(dsConfigurationNames) == dynlen(FWBM_DEVICE_CONFIG_NAME) && iIndexConfig == FWBM_DEVICE_REFERENCE_POSITION)
			{
				//Device. Necessary change the reference's name (the 4th element)
				sReferenceName = "";
				fwBenchmark_dpSetAttributes_getDeviceReferenceName(sConfiguration,dsDpeValue[iIndexDpe],sReferenceName);
				dsAttributesBunch[iIndexLoop] = (string)daConfigurationValues[iIndexConfig] + sReferenceName;
			}
			else
			{
				//Archive, original value or alert handling
				dsAttributesBunch[iIndexLoop] = daConfigurationValues[iIndexConfig];
			}
			
			//Build the DPs bunch
			dsDpBunch[iIndexLoop] = dsDpeValue[iIndexDpe] + dsConfigurationNames[iIndexConfig];
			
			//Index update
			_fwBenchmark_dpSetAttributes_BuildingBunchIndexUpdate(sConfiguration, iIndexLoop, iIndexDpe, iIndexConfig);
						
		}
				
		//Some configuration needs a reconstruction of their DPEs
		_fwBenchmark_dpSetAttributes_RebuildDpe(sConfiguration, dsDpeValue, iIndexDpe);
		
		//perform the actual dpSet
		float fElaspedTime = 0;
		if (_fwBenchmark_dpSetAttributes_CoreHelper(dsDpBunch,dsAttributesBunch, sConfiguration, fElaspedTime) != FWBM_PVSS_SUCCESS)
		{
			return FWBM_PVSS_ERROR;
		}
		fTotalTime = fTotalTime + fElaspedTime;	
		
	}
	
	//and now do for the rest of the DP which can't make a full bunch
	if(iNumAttributes>=iNumIteration*iBunchSize + 1)
	{	
		//build the bunch
		dynClear(dsDpBunch);
		dynClear(dsAttributesBunch);
		int iIndexStart = iNumIteration*iBunchSize + 1;		
		for( int j = iIndexStart; j <= iNumAttributes; j++)
		{
			int iIndexLoop = (j-iIndexStart)+1;
			//Build the attributes value bunch
			string sReferenceName = "";
			if(dynlen(dsConfigurationNames) == dynlen(FWBM_DEVICE_CONFIG_NAME) && iIndexConfig == FWBM_DEVICE_REFERENCE_POSITION)
			{
				//Device. Necessary change the reference's name (the 4th element)
				sReferenceName = "";
				fwBenchmark_dpSetAttributes_getDeviceReferenceName(sConfiguration,dsDpeValue[iIndexDpe],sReferenceName);
				dsAttributesBunch[iIndexLoop] = (string)daConfigurationValues[iIndexConfig] + sReferenceName;
			}
			else
			{
				//Archive, original value or alert handling
				dsAttributesBunch[iIndexLoop] = daConfigurationValues[iIndexConfig];
			}
			
			//Build the DPs bunch
			dsDpBunch[iIndexLoop] = dsDpeValue[iIndexDpe] + dsConfigurationNames[iIndexConfig];
			
			//Index update
			_fwBenchmark_dpSetAttributes_BuildingBunchIndexUpdate(sConfiguration, iIndexLoop, iIndexDpe, iIndexConfig);
		}
		
		//Some configuration needs a reconstruction of their DPEs
		_fwBenchmark_dpSetAttributes_RebuildDpe(sConfiguration, dsDpeValue, iIndexDpe);
		
		//perform the actual dpSet
		if ( dynlen(dsDpBunch))
		{
			float fElaspedTime = 0;
			if (_fwBenchmark_dpSetAttributes_CoreHelper(dsDpBunch, dsAttributesBunch, sConfiguration, fElaspedTime) != FWBM_PVSS_SUCCESS)
			{
				return FWBM_PVSS_ERROR;
			}
			fTotalTime = fTotalTime + fElaspedTime;
		}
		
	}    
        
	//and finally store the result
	daResult[1] = iBunchSize; 
	daResult[2] = fTotalTime;
	
	return FWBM_PVSS_SUCCESS;
}

/** 
 * Reconstruction of DPEs (Delete and create) if the configuration require it.
 * @param sConfiguration	(string)	IN 
 * @param dsDpeValue	(dyn_string)	IN 
 * @param iIndexDpe	(int)	IN 
 * @return value of type 'int'
 */
private int _fwBenchmark_dpSetAttributes_RebuildDpe(string sConfiguration, dyn_string dsDpeValue, int iIndexDpe)
{
	//if alarm, delete the DPEs and create again
	if(sConfiguration == FWBM_CONFIG_ALERTHDL)
	{
		dyn_string split;
		string sDpReCreate;
		for(int k=1;k<=iIndexDpe - 1;k++)
		{			
			//dist_3:fw-AlertHandling-1-1-1-1.
			split = strsplit(dsDpeValue[k], ":");	//chose the part right of :
			sDpReCreate = strrtrim(split[2],".");	//delete the dot
			
			dpDelete(sDpReCreate);					//delete the DPE
			dpCreate(sDpReCreate, FWBM_DPTYPE);		//create the DPE with the same name
		}
	}
	return FWBM_PVSS_SUCCESS;
}

/** 
 * Update the index of DPEs and the Configuration to access correctly in the bunch building
 * @param sConfiguration	(string)	IN 
 * @param iIndexLoop	(int)	IN 
 * @param iIndexDpe	(int)	IN/OUT 
 * @param iIndexConfig	(int)	IN/OUT 
 * @return value of type 'int'
 */
private int _fwBenchmark_dpSetAttributes_BuildingBunchIndexUpdate(string sConfiguration, int iIndexLoop, int &iIndexDpe, int &iIndexConfig)
{
	switch(sConfiguration)
	{
		case FWBM_CONFIG_ORIGINALVALUE:
			//Original value
			iIndexDpe++;
			iIndexConfig = 1;
			break;
		
		case FWBM_CONFIG_ARCHIVE:
			//Archive
			if(iIndexConfig == dynlen(FWBM_ARCHIVE_CONFIG_NAME))
			{
				iIndexDpe++;
				iIndexConfig = 1;
			}
			else
			{
				iIndexConfig++;
			}
			break;
			
		case FWBM_CONFIG_ALERTHDL:
			//Alert Handling
			if(iIndexConfig == dynlen(FWBM_ALERTHDL_CONFIG_NAME))
			{
				iIndexDpe++;
				iIndexConfig = 1;
			}
			else
			{
				iIndexConfig++;
			}
			break;
			
		case FWBM_CONFIG_DRIVER_SIM:
		case FWBM_CONFIG_DRIVER_S7:
		case FWBM_CONFIG_DRIVER_MODBUS:
		case FWBM_CONFIG_DRIVER_IEC:
			//Devices
			if(iIndexConfig == dynlen(FWBM_DEVICE_CONFIG_NAME))
			{
				iIndexDpe++;
				iIndexConfig = 1;
			}
			else
			{
				iIndexConfig++;
			}
			break;
		
		default:
			return FWBM_PVSS_ERROR;
				
	}
	
	return FWBM_PVSS_SUCCESS;	
}
 
/** 
 * Helper function to dpSetAttributes_Core to actually do the dpGet and measure the time it takes
 * 
 * @param dsDp	(dyn_string)	IN 
 * @param sConfiguration	(string)	IN 
 * @param fElapsedTime	(float)	IN/OUT 
 * @return value of type 'int'
 */
private int _fwBenchmark_dpSetAttributes_CoreHelper( dyn_string dsDp, dyn_anytype daAttribute, string sConfiguration, float &fElapsedTime)
{	
	//Rename the DPNames with the parameters to obtain and put a value
	//In _original.._value there is only a attribute per DPE.
	//In other case, the new dsDp will be as longer as the old dsDp * iNumberAttributes
	
	time tStart, tEnd;
 	tStart = getCurrentTime();
	if ( dpSetWait(dsDp, daAttribute) != 0)
	{
		fwBM_log( FWBM_ERROR_LOG, __FUNCTION__, "Error while performing a dpSetWait on "+dynlen(dsDp)+" parameters.");
		return FWBM_PVSS_ERROR;
	}
	
	tEnd = getCurrentTime();
	fElapsedTime =  fwBenchmark_Util_computeElapsedTime( tStart, tEnd);
	
	return FWBM_PVSS_SUCCESS;
}

/** 
 * 
 * Get the reference name of a devices, in function of the standard name
 * Example for a reference = 65535
 * Simulator driver, "SIM.65535"
 * Simatic S7 Driver, "AG45.DB1.DBD65535"
 * Modbus Driver, "M.1.5.65535"
 * IEC Driver, "IECTARGET-3.6.5.5.3.5"
 * 
 * @param sConfiguration	(string)	IN 
 * @param sDpName	(string)	IN 
 * @param sReferenceName	(string)	IN/OUT 
 * @return value of type 'int'
 */
int fwBenchmark_dpSetAttributes_getDeviceReferenceName(string sConfiguration, string sDpName, string &sReferenceName)
{
	dyn_string split;
	string sRef;
	switch(sConfiguration)
	{
		case FWBM_CONFIG_DRIVER_SIM:
			//The DPE name is usually "dist_1:fw-SimaticS7Driver-1-1-1-1.". Split by "-" and choose the third vector (1-1-1-1.)
			split = strsplit(sDpName,"-");
			sReferenceName = split[3];
			break;
		
		case FWBM_CONFIG_DRIVER_S7:
			split = strsplit(sDpName,"-");
			sRef = split[3];
			sReferenceName = "DB1.DBD" + sRef;
			break;
		
		case FWBM_CONFIG_DRIVER_MODBUS:
			split = strsplit(sDpName,"-");
			sRef = split[3];
			sReferenceName = "1.5." + sRef;
			break;
		
		case FWBM_CONFIG_DRIVER_IEC:
			split = strsplit(sDpName,"-");
			
			//Obtain the unit, tens, hundreds and thousands for the reference name
			int iRef = (int)split[3];
			int iUnits, iTens, iHundreds, iThousands, iThousands2;
			iUnits = iRef%10;
			iTens = 		(iRef/10)%10;
			iHundreds = 	(iRef/100)%10;
			iThousands = 	(iRef/1000)%10;
			iThousands2=	(iRef/10000)%10;
			sReferenceName = "3." + iThousands2 + "." + iThousands + "." + iHundreds + "." + iTens + "." + iUnits;
			
			break;
		
		default:
			//There isn't a reference name (It's not a device)
			sReferenceName = "";
			return FWBM_PVSS_SUCCESS;
	}
	
	return FWBM_PVSS_SUCCESS;
}
