#uses "fwBenchmark/fwBenchmark_Machine.ctl"
/*
 * 
 * This library contains the support/common functions
 * to all benchmarks
 * 
 * jean-charles.tournier@cern.ch
 * CERN - EN/ICE/SCD
 * 
 * July 9th, 2015
 * 
 */
 
 #uses "fwBenchmark/fwBenchmark_Const.ctl"
 
 //global variable used to store the error and warning messages
 //so they can be retrieved all at once when creating the 
 //benchmark reports
 private dyn_string fwLogStorage; 
 
 
 /** 
 *
 * Clear the global log storage
 * It should be called at the beginning 
 * of all benchmark
 *  
 * @return value of type 'void'
 */
 void fwBenchmark_Util_clearLog()
 {
 	dynClear(fwLogStorage);
 }
 
 
 /** 
 *
 *	Return the content of the global log storage
 * which accumulated all log messages displayed during
 * the benchmark execution. It should be called when 
 * generating the benchmark report 
 *  
 * @return value of type 'dyn_string'
 */
 dyn_string fwBenchmark_Util_getLog()
 {
 	return fwLogStorage;
 }
 
 
 /** 
 *
 * Build the common parts of all benchmark reports
 *  
 * @return value of type 'dyn_string'
 */
 dyn_string fwBenchmark_Util_getReportCommon( string sBenchmarkTitle)
 {
 	dyn_string dsMachineCharac = fwBenchmark_Machine_getCharacteristics();
 	dyn_string dsContent;
 	
 	dynAppend(dsContent, FWBM_NEW_SECTION);
 	dynAppend(dsContent, "Date/Time of the report generation :"+getCurrentTime());
 	dynAppend(dsContent, FWBM_NEW_SECTION);
 	dynAppend(dsContent, "Benchmark name: "+sBenchmarkTitle);
 	dynAppend(dsContent, "Number of repetitions: "+FWBM_NUM_ROUND);
 	dynAppend(dsContent, FWBM_NEW_SECTION);
 	dynAppend(dsContent, dsMachineCharac);
 	dynAppend(dsContent, FWBM_NEW_SECTION);
 	dynAppend(dsContent, "\tWARNING and ERROR Mesages");
 	dynAppend(dsContent, fwBenchmark_Util_getLog());
 	dynAppend(dsContent, FWBM_NEW_SECTION);
 	dynAppend(dsContent, fwBenchmark_Util_retrieveResourcesMonitoring());
 	dynAppend(dsContent, FWBM_NEW_SECTION);
 	return dsContent;
 }
 
/** 
 *
 * Given two times, return the elapsed time in seconds with millisecond precisions
 *  
 * @param start	(time)	IN	: start time
 * @param stop	(time)	IN	: end time
 * @return value of type 'float': difference between stop and start time expressed in second.milliseconds
 */
float fwBenchmark_Util_computeElapsedTime( time start, time stop)
{
  float delta;
  
  
  int a = period( start);
  int b = period( stop);
  int tot;
    
  int mil_a = milliSecond( start);
  int mil_b = milliSecond( stop);
  float mil_tot;
  
  if (mil_b < mil_a)
  {
    mil_b += 1000;
    --b;
  }
  
  mil_tot = mil_b - mil_a;
  
  tot = b - a;
  
  
  delta = tot + mil_tot/1000;
   
  return delta;
}


/** 
 *	Simply returns a string representing the current time
 * @return value of type 'string'
 */
string fwBenchmark_Util_getPrintableTime()
{
    time t = getCurrentTime();
    string st = formatTime("%Y.%m.%d %H:%M:%S.", t, "%03d");
    return st;
}


/** 
 *	Manage the print/display of the log messages
 *  
 * @param iLogLevel	(int)	IN	can be FWBM_ERROR_LOG, FWBM_WARNING_LOG or FWBM_INFO_LOG
 * @param sOriginatorFunction	(string)	IN	name of the function filled in by the caller using __FUNCTION__
 * @param ... 	arbitrary number of parameters: list of parameters to be displayed in single line
 * @return value of type 'void'
 */
void fwBM_log(int iLogLevel, string sOriginatorFunction, ...)
{
	//get the time
	string sTime = fwBenchmark_Util_getPrintableTime();
	
	//build the message to display
	string sMessage = "["+sOriginatorFunction+"]";
	va_list parameters;
	int iNumParam = va_start(parameters);
	for( int i=1; i<=iNumParam; i++)
	{
		sMessage += va_arg(parameters) + " ";
	}
	va_end(parameters);
	
	//get the log level in string
	string sLogLevel = "INFO";
	string sColor = "black";
	if ( iLogLevel == FWBM_WARNING_LOG)
	{
		sLogLevel = "WARNING";
		sColor = "STD_value_default";
		dynAppend( fwLogStorage, "["+sLogLevel+"] " + sMessage); //appends to the storage log so it can be dumped to the report
	} else if ( iLogLevel == FWBM_ERROR_LOG)
	{
		sLogLevel = "ERROR";
		sColor = "red";
		dynAppend( fwLogStorage, "["+sLogLevel+"] " + sMessage); //appends to the storage log so it can be dumped to the report
	}
	
	//if log table exist, print
	if ( isFunctionDefined("getShape"))
	{
		shape shTableLog = getShape(FWBM_LOG_TABLE);
		if ( shapeExists(FWBM_LOG_TABLE))
		{
			shTableLog.appendLine("type", sLogLevel, "ts", sTime, "msg", sMessage);
			int row_num = shTableLog.lineCount();
  			shTableLog.cellForeColRC( row_num-1, "ts", sColor);
  			shTableLog.cellForeColRC( row_num-1, "msg", sColor);
		}	
	}
	 
	
	
	//in any case, display it in the log
	DebugTN("["+sLogLevel+"]", sMessage);
	
}

/**
  Write the content of a dyn_dyn_string to a file
  
  @param
    string sFileName: name of the file to be created. If the file already exists it will be erased
    dyn_string dsContentFile: content of the file to be dumped
  @return
    FWBM_PVSS_SUCCESS in case of sucess, i.e. the file is created
  **/
int fwBenchmark_Util_writeFile( string sFileName, dyn_string ddsFileContent)
{
  file fFile = fopen( sFileName, "w");
    
  dyn_errClass deErr = getLastError();

  if( dynlen(deErr))
  {
    fwBM_log( FWBM_ERROR_LOG, __FUNCTION__, "Error trying to write to file "+sFileName);
    fwBM_log( FWBM_ERROR_LOG, __FUNCTION__, deErr);
    return FWBM_PVSS_ERROR;
  }  
  
  int iNumLines = dynlen(ddsFileContent);
  for( int i=1; i<=iNumLines; i++)
  {
    string sLine = ddsFileContent[i];
    fputs( sLine+"\n", fFile);
  }  
  
  fclose(fFile);
  
  return FWBM_PVSS_SUCCESS;
}

/** 
 * 
 * Simply read a file and dump its content to a dyn_string
 * 
 * @param file_name	(string)	IN	name of the file to read
 * @return value of type 'dyn_string' content of the file read
 */
dyn_string fwBenchmark_Util_readFile( string file_name)
{
  dyn_string file_content;
  
  file f;
  int r;
  
  f = fopen( file_name, "r");
  int err = ferror(f);
  if (err!=0)
  {
    fwBM_log( FWBM_ERROR_LOG, __FUNCTION__, "Can not open file "+file_name); 
    fwBM_log( FWBM_ERROR_LOG, __FUNCTION__, "error code: "+err); 
    return file_content;  
  } else
  {
    while(feof(f)==0)
    {
      string tmp;
      fgets( tmp, 1054, f);
      strreplace(tmp, "\n", "");
      dynAppend( file_content, tmp);     
    }
  }
  fclose(f);
  return file_content;
}


/** 
 * Builds a string representing the date and time (down to the second). 
 * It is used to prefix the name of the benchmark result file
 * 
 * 
 * @return value of type 'string' representing the current date and time, e.g. 2015_07_14-09_33_29-
 */
string fwBenchmark_Util_getDateTimePrefix()
{
	time tCurrentTime = getCurrentTime();
	
	return year(tCurrentTime)+"_"+month(tCurrentTime)+"_"+day(tCurrentTime)+"-"
			+hour(tCurrentTime)+"_"+minute(tCurrentTime)+"_"+second(tCurrentTime)+"-";
}


//Global variables used by the resources monitoring functions
dyn_string _dsResourceReportFiles;
dyn_string _dsResourceStopFiles;
dyn_string _dsManagerToMonitor = makeDynString( "WCCILevent", "WCCILdata");
 /** 
 *	Starts the tools to monitor the CPU and memory usage
 *  of the core processes of WCCOA
 *  
 * @return value of type 'void'
 */
int fwBenchmark_Util_startResourcesMonitoring()
{
	if (_UNIX)
	{
		dynClear(_dsResourceReportFiles);
		dynClear(_dsResourceStopFiles);
		int iNumManagers=dynlen(_dsManagerToMonitor);
		for( int i=1; i<=iNumManagers; i++)
		{
			string sFileReport = tmpnam();
			string sFileStop = tmpnam();
			system(getPath(SOURCE_REL_PATH,"fwBenchmark/fwBenchmark_MonitorResources.sh")
					+" "+sFileReport+" "+sFileStop+" "+_dsManagerToMonitor[i]+"&"
			   	);
			dynAppend(_dsResourceReportFiles, sFileReport);
			dynAppend(_dsResourceStopFiles, sFileStop);
			delay(1);
		}			
	}
	else if( _WIN32)
	{
		dynClear(_dsResourceReportFiles);
		dynClear(_dsResourceStopFiles);
		int iNumManagers=dynlen(_dsManagerToMonitor);
		for( int i=1; i<=iNumManagers; i++)
		{
			string sFileReport = tmpnam();
			string sFileStop = tmpnam();
			string pathscriptfile = getPath(SOURCE_REL_PATH,"fwBenchmark\\fwBenchmark_MonitorResources.bat");
			//strreplace(pathscriptfile, "/", "\\"); 
			
			//'start "titlewindow" /B' means initialize the script without creating a new windows
			//From the WCCOActrl works but it doesn't work from the WCCOAui, so, we will see the windows minimized.
			system("start \"MonitorResources\" /min " + pathscriptfile
					+" " + sFileReport +" "+ sFileStop + " " + _dsManagerToMonitor[i]
			   	); 	
			dynAppend(_dsResourceReportFiles, sFileReport);
			dynAppend(_dsResourceStopFiles, sFileStop);
			delay(1);
		}	
	}
	else
	{
		fwBM_log( FWBM_ERROR_LOG, __FUNCTION__, "Function not implemented for this OS.");
		return FWBM_PVSS_ERROR;
	}
	
	delay(4); //put a delay to make sure all managers are back to 0% of usage
	return FWBM_PVSS_SUCCESS;
}


/** 
 *	This function stops the resources monitoring started by 
 *  fwBenchmark_Util_startResourcesMonitoring 
 *  
 * @return value of type 'dyn_string'
 */
int fwBenchmark_Util_stopResourcesMonitoring()
{
	delay(2); //put a delay to make sure all managers are back to 0% of usage
	int iMAXERROR=7;
	
	if (_UNIX)
	{		
		int iNumManagers=dynlen(_dsResourceStopFiles);
		for( int i=1; i<=iNumManagers; i++)
		{
			
			int iErrorCount = 0;
			//Create the stop file
			system("touch "+_dsResourceStopFiles[i]+"&"); //by simply creating the file, it stops the script fwBenchmark_MonitorResources.sh
			delay(2);	//Wait until the scripts knows that this file has been created (ghost processes bug fixed!)
			while( !isfile(_dsResourceStopFiles[i]))
			{
				delay(1); //wait until the stop file is really created
				//The program has iMAXERROR opportunities to create the stop file, else, it will report an error.
				iErrorCount++;
				if(iErrorCount >= iMAXERROR)
				{
					fwBM_log( FWBM_ERROR_LOG, __FUNCTION__, "Error creating the stop file by the script .sh. Temporal file destination: "+ _dsResourceStopFiles[i]);
					//system("killall -9 fwBenchmark_MonitorResources.sh &");
					//return FWBM_PVSS_ERROR;
					break;
				}
			}
			iErrorCount=0;
			
			///Check the report file has been created by the script .sh
			while( !isfile(_dsResourceReportFiles[i])) 
			{
				delay(1); //wait until the report file is really created
				//The program has iMAXERROR opportunities to create the report file, else, it will report an error.
				iErrorCount++;
				if(iErrorCount >= iMAXERROR)
				{
					fwBM_log( FWBM_ERROR_LOG, __FUNCTION__, "Error creating the report file by the script .sh. Temporal file destination: "+ _dsResourceReportFiles[i]);
					return FWBM_PVSS_ERROR;
				}
			}
			iErrorCount=0;
			
			//Delete the stop file created previously and check if it has been deleted correctly
			delay(1); //The script should have stopped, before removing the stop file. Check with "ps aux | grep fwBenchmark_MonitorResources.sh" ?
			remove(_dsResourceStopFiles[i]);
			if( isfile(_dsResourceStopFiles[i]))
			{
				delay(2); //wait and try to delete again. If is not possible, throw a warning (it is not critical)
				remove(_dsResourceStopFiles[i]);
				if( isfile(_dsResourceStopFiles[i]))
					fwBM_log( FWBM_WARNING_LOG, __FUNCTION__, "File _dsResourceStopFiles has not been removed. Destination:" + _dsResourceStopFiles[i]);
			}	
			
		}		
	} 
	else if( _WIN32)
	{
		int iNumManagers=dynlen(_dsResourceStopFiles);
		for( int i=1; i<=iNumManagers; i++)
		{
			int iErrorCount = 0;
			//Create the stop file
			system("start \"Stop batch script\" /b touch "+_dsResourceStopFiles[i]); //by simply creating the file, it stops the script fwBenchmark_MonitorResources.bat
			delay(2);	//Wait until the scripts knows that this file has been created
			while( !isfile(_dsResourceStopFiles[i]))
			{
				delay(1); //wait until the stop file is really created
				//The program has iMAXERROR opportunities to create the stop file, else, it will report an error.
				iErrorCount++;
				if(iErrorCount >= iMAXERROR)
				{
					fwBM_log( FWBM_ERROR_LOG, __FUNCTION__, "Error creating the stop file by the script .bat. Temporal file destination: "+ _dsResourceStopFiles[i]);
					//return FWBM_PVSS_ERROR;
					break;
				}
			}
			iErrorCount=0;

			//Check the report file has been created by the script .bat
			while( !isfile(_dsResourceReportFiles[i]))
			{
				delay(1); //wait until the report file is really created
				//The program has iMAXERROR opportunities to create the report file, else, it will report an error.
				iErrorCount++;
				if(iErrorCount >= iMAXERROR)
				{
					fwBM_log( FWBM_ERROR_LOG, __FUNCTION__, "Error creating the report file by the script .bat. Temp file: "+ _dsResourceReportFiles[i]);
					return FWBM_PVSS_ERROR;
				}
			}
			iErrorCount=0;
			
			//Delete the stop file created previously and check if it has been deleted correctly
			delay(1); //The script should have stopped, before removing the stop file. Check with "ps aux | grep fwBenchmark_MonitorResources.sh" ?
			remove(_dsResourceStopFiles[i]);
			if( isfile(_dsResourceStopFiles[i]))
			{
				delay(2); //wait and try to delete again. If is not possible, throw a warning (it is not critical)
				remove(_dsResourceStopFiles[i]);
				if( isfile(_dsResourceStopFiles[i]))
					fwBM_log( FWBM_WARNING_LOG, __FUNCTION__, "File _dsResourceStopFiles has not been removed. Destination:" + _dsResourceStopFiles[i]);
			}		
		}
	}
	else
	{
		fwBM_log( FWBM_ERROR_LOG, __FUNCTION__, "Function not implemented for this OS.");
		return FWBM_PVSS_ERROR;
	}
	return FWBM_PVSS_SUCCESS;	
}


/** 
 *	This function returns
 *  a dyn_string string containing the CPU and memory usage
 * 	of all manager requested
 *  
 * @return value of type 'dyn_string'
 */
dyn_string fwBenchmark_Util_retrieveResourcesMonitoring()
{
	dyn_string dsContent;
	
	if (_UNIX)
	{
		int iNumManagers=dynlen(_dsResourceStopFiles);
		for( int i=1; i<=iNumManagers; i++)
		{					
			//get the content of the report resources usage file 
			dynAppend( dsContent, FWBM_NEW_SECTION);
			dynAppend( dsContent, "Resources usage of "+_dsManagerToMonitor[i]);
			dynAppend( dsContent, "CPU (1s)\tRes. Memory");
			dyn_string dsReportContent = fwBenchmark_Util_readFile(_dsResourceReportFiles[i]);
			dynAppend( dsContent, dsReportContent);
				
			//Delete the temporal report file and check if it has been deleted correctly
			remove(_dsResourceReportFiles[i]);
			delay(1); //wait until the file is really deleted
			if( isfile(_dsResourceStopFiles[i]))
			{
				delay(1); //wait and try to delete again. If is not possible, throw a warning (it is not critical)
				remove(_dsResourceReportFiles[i]);
				if( isfile(_dsResourceReportFiles[i]))
					fwBM_log( FWBM_WARNING_LOG, __FUNCTION__, "_dsResourceReportFiles has not been deleted: " + _dsResourceReportFiles[i]);
			}	
		}
	} 
	else if( _WIN32)
	{
		int iNumManagers=dynlen(_dsResourceStopFiles);
		for( int i=1; i<=iNumManagers; i++)
		{					
			//get the content of the report resources usage file 
			dynAppend( dsContent, FWBM_NEW_SECTION);
			dynAppend( dsContent, "Resources usage of "+_dsManagerToMonitor[i]);
			dynAppend( dsContent, "CPU (1s)\tRes. Memory");
			dyn_string dsReportContent = fwBenchmark_Util_readFile(_dsResourceReportFiles[i]);
			dynAppend( dsContent, dsReportContent);
			
			//Delete the temporal report file and check if it has been deleted correctly
			remove(_dsResourceReportFiles[i]);
			delay(1); //wait until the file is really deleted
			if( isfile(_dsResourceStopFiles[i]))
			{
				delay(1); //wait and try to delete again. If is not possible, throw a warning (it is not critical)
				remove(_dsResourceReportFiles[i]);
				if( isfile(_dsResourceReportFiles[i]))
					fwBM_log( FWBM_WARNING_LOG, __FUNCTION__, "_dsResourceReportFiles has not been deleted: " + _dsResourceReportFiles[i]);
			}	
		}
	}
	else
	{
		fwBM_log( FWBM_ERROR_LOG, __FUNCTION__, "Function not implemented for this OS.");
	}
	
	return dsContent;
}


/** 
 *
 * 	Send an email to the list of specified recipients
 *  
 * @param dsTo	(dyn_string)	IN : list of recipients
 * @param sSubject	(string)	IN : Subject of the mail
 * @param sContent	(string)	IN : Content of the email
 * @param sFileAttach	(string)	IN : Name of the file to be attached
 * @return value of type 'void'
 */
public void fwBenchmark_Util_sendEmail( dyn_string dsTo, string sSubject, string sContent, string sFileAttach="")
{
  //if no recipient specified no need to continue
  if ( dynlen(dsTo) < 1)
  {
  	return;
  }
  
  string smtpServer = "cernmx.cern.ch";
  string sFrom = "fwBenchmark@cern.ch";
  string sBody = sContent;
  string sTo = (string)dsTo;
  strreplace( sTo, "|", "");
  string sCurrentTime = (string)getCurrentTime();
  sBody += "\n\nEmail sent from "+getHostname()+" at "+sCurrentTime;
  sBody += "\n\nThis is an automatically generated email. Please do not reply.";
  
  if (_WIN32)
  {
    strreplace(sBody, "\n", "\\n");  
  }
 
  string emailCommand = PVSS_PATH + "bin/sendEmail -f " + sFrom + " " +
                        "-t " + sTo + " " +
                        "-u " + "\"" + sSubject + "\"" + " " +
                        "-m " + "\"" + sBody + "\"" + " " +
                        "-s " + smtpServer;
  
   if( sFileAttach!= "")
   {
   	if ( isfile(sFileAttach))
   	{
   		emailCommand += " -a " + sFileAttach;
   	} else
   	{
   		fwBM_log( FWBM_ERROR_LOG, __FUNCTION__, "File does not exist - Not attached to email "+ sFileAttach);
   	}
   }
    
    
  int ret = system(emailCommand);
  return ;
}


